# -*- coding: utf-8 -*-
class Admin::ProblemGroupsController < AdminController

  access_control do
    allow :admin, :teacher
    deny anonymous
  end

  before_filter :prepare, :only => [:show, :edit, :update, :destroy]
  before_filter :prepare_problems, :only => [:show, :edit, :new]

  # GET admin_problem_groups_path
  def index
    @problem_groups = ProblemGroup.all(:order => 'position')
  end

  # GET admin_problem_group_path(:id)
  def show
  end

  # GET edit_admin_problem_group_path
  def edit
  end

  # PUT admin_problem_group_path(:id)
  def update
    @problem_group.problems = Problem.all(:conditions =>
                                          { :id => params[:problem_group].delete(:problems) })
    @problem_group.attributes = params[:problem_group]
    if @problem_group.save
      flash[:notice] = 'Problem group was successfully updated.'
      redirect_to :action => 'index'
    else
      render :action => 'edit'
    end
  end

  # GET new_admin_problem_group_path
  def new
    @problem_group = ProblemGroup.new
  end

  # POST admin_problem_groups_path
  def create
    problems = Problem.all(:conditions => { :id => params[:problem_group].delete(:problems) })
    @problem_group = ProblemGroup.new(params[:problem_group])
    @problem_group.problems = problems
    if @problem_group.save
      flash[:notice] = 'Problem group was successfully created.'
      redirect_to :action => 'index'
    else
      render :action => 'new'
    end
  end

  # DELETE admin_problem_group_path(:id)
  def destroy
    @problem_group.destroy
    flash[:notice] = 'Problem group was successfully deleted.'
    redirect_to :action => 'index'
  end

  def sort
    ProblemGroup.transaction do
      params[:position].each.with_index do |id, index|
        ProblemGroup.find(id).update_attribute(:position, index + 1)
      end
      respond_to do |format|
        format.json{ render :json => { :success => true, :message => "success" } }
      end
    end
  end

  private

  def prepare
    @problem_group = ProblemGroup.find(params[:id])
  end

  def prepare_problems
    @problems = Problem.all(:include => :unit,
                            :order   => "units.position, level")
  end

end
