# -*- coding: utf-8 -*-
class Admin::ProblemsController < AdminController

  access_control do
    allow :admin, :teacher
    deny anonymous
  end

  before_filter :prepare, :only => [:show, :edit, :update, :destroy]

  # GET /admin/problems
  # GET admin_problems_path
  def index
    @problems = Problem.paginate(:page    => params[:page],
                                 :include => :unit,
                                 :order   => 'units.position ASC, level ASC')
  end

  # GET /admin/problem/:id
  # GET admin_problem_path(id)
  def show
    @form = @form_klass.new(:unit_id => @problem.unit_id, :level => @problem.level)
    @form.set_options(@problem.options)
  end

  # GET /admin/problems/new
  # GET new_admin_problem_path
  def new
    unit = Unit.find_by_id(params[:unit_id])
    if unit
      @form_klass = unit.form_class
      @form = @form_klass.new(:unit_id => unit.id)
      if request.xhr?
        render :template => "/admin/problems/new_form", :layout => false
      else
        render :template => "/admin/problems/new_form"
      end
    else
      @units = Unit.all(:order => "position")
    end
  end

  # POST /admin/problems
  # POST admin_problems_path
  def create
    unit = Unit.find_by_id(params[:problem][:unit_id])
    if unit
      @form_klass = unit.form_class
      @form = @form_klass.new(params[:problem])
    end
    if @form && @form.valid? && Problem.create!(@form.attributes_for_model)
      flash[:notice] = _("Problem was successfully created.")
      redirect_to admin_problems_path
    else
      render :template => "/admin/problems/new_form"
    end
  end

  # GET edit_admin_problem_path(id)
  def edit
    @form = @form_klass.new(:unit_id      => @problem.unit.id,
                            :level        => @problem.level,
                            :lock_version => @problem.lock_version)
    @form.set_options(@problem.options)
  end

  # PUT /admin/problems/:id
  def update
    @form = @form_klass.new(params[:problem])
    if @form.valid? && @problem.update_attributes!(@form.attributes_for_model)
      flash[:notice] = "問題の設定を更新しました。"
      redirect_to admin_problems_path
    else
      render :template => "/admin/problems/edit"
    end
  end

  # DELETE /admin/problems/:id
  def destroy
    @problem.destroy
    flash[:notice] = "問題の設定を削除しました。"
    redirect_to admin_problems_path
  end

  private

  def prepare
    @problem = Problem.find(params[:id])
    @form_klass = @problem.unit.form_class
  end

end
