class Admin::UnitsController < AdminController

  access_control do
    allow :admin
    deny anonymous
  end

  before_filter :prepare, :only => [:show, :edit, :update, :destroy]

  def index
    @units = Unit.all(:order => 'position')
  end

  def show
  end

  def edit
  end

  def update
    @unit.attributes = params[:unit]
    if @unit.save
      flash[:notice] = _('Unit was successfully updated.')
      redirect_to :action => 'index'
    else
      render :action => 'edit'
    end
  end

  def new
    @unit = Unit.new
  end

  def create
    @unit = Unit.new(params[:unit])
    if @unit.save
      flash[:notice] = 'Unit was successfully created.'
      redirect_to :action => 'index'
    else
      render :action => 'new'
    end
  end

  def destroy
    @unit.destroy
    flash[:notice] = _('Unit was successfully deleted.')
    redirect_to :action => 'index'
  end

  def sort
    Unit.transaction do
      params[:position].each.with_index do |id, index|
        Unit.find(id).update_attribute(:position, index + 1)
      end
      respond_to do |format|
        format.json{ render :json => { :success => true, :message => "success" } }
      end
    end
  end

  private

  def prepare
    @unit = Unit.find(params[:id])
  end

end
