# -*- coding: utf-8 -*-
class Admin::UsersController < AdminController

  access_control do
    allow :admin
    deny anonymous
  end

  before_filter :prepare, :only => [:show, :edit, :update]

  # GET /admin/users
  # GET admin_users_path
  def index
    @users = User.paginate(:page  => params[:page],
                           :order => 'admission_date ASC')
  end

  # GET /admin/users/:id
  # GET admin_user_path(id)
  def show
  end

  # NOTE 必要？
  def new
  end

  # NOTE 必要？
  def create
  end

  # GET edit_admin_user_path(id)
  def edit
    @form = UserEditForm.new(:name => @user.name,
                             :email => @user.email,
                             :admission_date => @user.admission_date,
                             :lock_version => @user.lock_version)
  end

  def update
    role = params[:user].delete(:role)
    @form = UserEditForm.new(params[:user])
    if @form.valid? && set_role(role) && @user.update_attributes!(@form.attributes)
      flash[:notice] = "ユーザ情報を更新しました。"
      redirect_to admin_users_path
    else
      render :template => "/admin/users/edit"
    end
  end

  # DELETE /admin/user/:id
  def destroy
    User.destroy(params[:id])
    flash[:notice] = "ユーザを削除しました。"
    redirect_to admin_users_path
  end

  # DELETE /admin/users/destroy_selected
  def destroy_selected
    user_ids = params[:user_ids].map(&:to_i)
    begin
      User.transaction do
        User.destroy(user_ids)
      end
    rescue
      flash[:error] = "ユーザを削除中にエラーが発生しました。"
      redirect_to admin_users_path
      return
    end
    flash[:notice] = "#{user_ids.size} 件の指定されたユーザを削除しました。"
    redirect_to admin_users_path
  end

  private

  def prepare
    @user = User.find(params[:id])
  end

  def set_role(role)
    @user.has_no_roles!
    @user.has_role!(role)
    true
  end
end
