# -*- coding: utf-8 -*-
class ProblemsController < ApplicationController

  InvalidSubmitToken = Class.new(StandardError)

  rescue_from InvalidSubmitToken do
      flash[:error] = '解答済みの問題に解答することは出来ません。'
      redirect_to :action => 'quiz', :id => params[:problem_group_id]
  end

  before_filter :authentication_required
  before_filter :prepare_submit_token, :only => [:quiz]
  before_filter :verify_submit_token, :only => [:answer]

  # GET problem/quiz/:id
  def quiz
    @group = ProblemGroup.find(params[:id])
    @problems = @group.problems.map{|problem| [problem, problem.generate] }
  end

  # POST problems/answer
  def answer
    problems = params[:problems]
    answers  = params[:answers]
    problem_ids = params[:problem_ids]
    problem_group_id = params[:problem_group_id]
    report = nil
    ActiveRecord::Base.transaction do
      report = current_user.reports.build(:problem_group_id => problem_group_id)
      report.save!
      problems.zip(answers, problem_ids) do |problem, answer, problem_id|
        report.learning_results.build(:problem_expression => problem,
                                      :answer => answer,
                                      :problem_id => problem_id)
      end
      report.save!
    end
    session[:double_submit_token] << params[:submit_token]
    session[:double_submit_token].shift if session[:double_submit_token].size > 5
    redirect_to report_path(report)
  end

  private

  def prepare_submit_token
    @submit_token = SecureRandom.hex(20)
    session[:prepared_submit_token] ||= []
    session[:prepared_submit_token] << @submit_token
    session[:prepared_submit_token].shift if session[:prepared_submit_token].size > 5
  end

  def verify_submit_token
    session[:double_submit_token] ||= []
    if !session[:prepared_submit_token].include?(params[:submit_token]) ||
        session[:double_submit_token].include?(params[:submit_token])
      raise InvalidSubmitToken
    end
  end

end
