class ReportsController < ApplicationController

  before_filter :authentication_required
  before_filter :prepare_user
  before_filter :prepare_problem_groups, :only => [:index, :range]

  def index
    year  = params[:year]  || Date.today.year
    month = params[:month] || Date.today.month
    day   = params[:day]   || Date.today.day
    @date = Date.parse("#{year}-#{month}-#{day}")
    @problem_group_id = params[:problem_group_id]
    reports = Report.by_user(@user).by_date(@date)
    reports = reports.by_problem_group(@problem_group_id) if @problem_group_id
    respond_to do |format|
      format.html{
        @reports = reports.
        paginate(:page => params[:page],
                 :per_page => 20,
                 :include => %w[learning_results problem_group],
                 :order => 'created_at ASC')
      }
      format.pdf{
        output = SummaryReport.new(params, @user).to_pdf(reports)
        send_data output, :filename => "report.pdf"
      }
    end
  end

  def range
    @start_date, @end_date = params[:range].split('-').map{|v| Date.parse(v) }
    @problem_group_id = params[:problem_group_id]
    reports = Report.by_user(@user).by_range(@start_date, @end_date)
    reports = reports.by_problem_group(@problem_group_id) if @problem_group_id
    respond_to do |format|
      format.html{
        @reports = reports.
        paginate(:page => params[:page],
                 :per_page => 20,
                 :include => %w[learning_results problem_group],
                 :order => 'created_at ASC')
      }
      format.pdf{
        output = SummaryReport.new(params, @user).to_pdf(reports)
        send_data output, :filename => "report.pdf"
      }
    end
  end

  def show
    @date = Date.today
    @reports = @user.reports.by_date(Date.today)
    @report = @user.reports.first(:conditions => { :id => params[:id] })
    @results = @report.learning_results
    @problem_group_id = @report.problem_group_id
    respond_to do |format|
      format.html
      format.pdf{
        output = SimpleReport.new(params, @user).to_pdf(@report, @results)
        send_data output, :filename => "report.pdf"
      }
    end
  end

  def search
    @date       = Date.parse(params[:date]) if params[:date]
    @start_date = params[:start_date].tr('-', '') if params[:start_date]
    @end_date   = params[:end_date].tr('-', '') if params[:end_date]
    @problem_group_id = params[:problem_group_id]
    case
    when @date
      o = { :action => "index", :year => @date.year, :month => @date.month, :day => @date.day }
      o.merge!({:problem_group_id => @problem_group_id }) unless @problem_group_id.blank?
      redirect_to(o)
    when @start_date && @end_date
      o = { :action => "range", :start => @start_date, :end => @end_date }
      o.merge!({ :problem_group_id => @problem_group_id }) unless @problem_group_id.blank?
      redirect_to(o)
    end
  end

  private

  def prepare_user
    @user = current_user
  end

  def prepare_problem_groups
    @problem_groups = ProblemGroup.all
    o = Object.new
    def o.id; nil end
    def o.label; '' end
    @problem_groups.unshift(o)
  end

end
