# == Schema Information
# Schema version: 20100309071953
#
# Table name: active_forms
#
#  unit_id              :
#  level                :integer
#  amount               :integer
#  lock_version         :integer
#  operators_plus       :boolean
#  operators_minus      :boolean
#  operators_times      :boolean
#  numerator_term_min   :integer
#  numerator_term_max   :integer
#  denominator_term_min :integer
#  denominator_term_max :integer
#

class ComplexFractionalArithmeticEditForm < ActiveForm
  acts_as_problem :operators => %w[plus minus times]

  column :numerator_term_min, :type => :integer
  column :numerator_term_max, :type => :integer
  column :denominator_term_min, :type => :integer
  column :denominator_term_max, :type => :integer

  validates_presence_of :numerator_term_min, :type => :integer
  validates_presence_of :numerator_term_max, :type => :integer
  validates_presence_of :denominator_term_min, :type => :integer
  validates_presence_of :denominator_term_max, :type => :integer

  with_options :only_integer => true, :allow_blank => true do |problem|
    problem.validates_numericality_of :numerator_term_min, :greater_than_or_equal_to => 1
    problem.validates_numericality_of :numerator_term_max, :greater_than_or_equal_to => 1
    problem.validates_numericality_of :denominator_term_min, :greater_than_or_equal_to => 1
    problem.validates_numericality_of :denominator_term_max, :greater_than_or_equal_to => 1
  end

  def options_hash
    {
      :numerator_term_min   => numerator_term_min,
      :numerator_term_max   => numerator_term_max,
      :denominator_term_min => denominator_term_min,
      :denominator_term_max => denominator_term_max,
      :operators            => operators,
    }
  end

  def set_options(options)
    hash = ActiveSupport::JSON.decode(options)
    self.numerator_term_min   = hash["numerator_term_min"]
    self.numerator_term_max   = hash["numerator_term_max"]
    self.denominator_term_min = hash["denominator_term_min"]
    self.denominator_term_max = hash["denominator_term_max"]
    self.operators            = hash["operators"] if hash["operators"]
  end

  def validate
    check_min_max(:numerator_term_min, :numerator_term_max)
    check_min_max(:denominator_term_min, :denominator_term_max)
    check_operators()
  end

end
