# == Schema Information
# Schema version: 20100309071953
#
# Table name: active_forms
#
#  unit_id            :
#  level              :integer
#  amount             :integer
#  lock_version       :integer
#  operators_plus     :boolean
#  operators_minus    :boolean
#  operators_times    :boolean
#  operators_fraction :boolean
#  term_number        :integer
#  min                :integer
#  max                :integer
#  fractional_mode    :boolean
#

class ComplexNumberArithmeticEditForm < ActiveForm
  acts_as_problem :operators => %w[plus minus times fraction], :term_number => true

  column :min, :type => :integer
  column :max, :type => :integer
  column :fractional_mode, :type => :boolean

  validates_presence_of :min
  validates_presence_of :max

  with_options :only_integer => true, :allow_blank => true do |problem|
    problem.validates_numericality_of :min, :greater_than_or_equal_to => 0
    problem.validates_numericality_of :max, :greater_than_or_equal_to => 0
  end

  def options_hash
    {
      :min             => min,
      :max             => max,
      :fractional_mode => fractional_mode,
      :term_number     => term_number,
      :operators       => operators,
    }
  end

  def set_options(options)
    hash = ActiveSupport::JSON.decode(options)
    self.min             = hash["min"]
    self.max             = hash["max"]
    self.fractional_mode = hash["fractional_mode"]
    self.term_number     = hash["term_number"] if hash["term_number"]
    self.operators       = hash["operators"] if hash["operators"]
  end

  def validate
    check_min_max(:min, :max)
    check_operators()
  end

end
