# == Schema Information
# Schema version: 20100309071953
#
# Table name: active_forms
#
#  unit_id      :
#  level        :integer
#  amount       :integer
#  lock_version :integer
#  order_min    :integer
#  order_max    :integer
#  x            :
#  factor_minus :boolean
#  factor_min   :integer
#  factor_max   :integer
#

class ExpansionEditForm < ActiveForm
  acts_as_problem

  column :order_min, :type => :integer
  column :order_max, :type => :integer
  column :x
  column :factor_minus, :type => :boolean
  column :factor_min,   :type => :integer
  column :factor_max,   :type => :integer

  validates_presence_of :order_min
  validates_presence_of :order_max
  with_options :only_integer => true, :allow_blank => true do |problem|
    problem.validates_numericality_of :order_min, :greater_than => 1
    problem.validates_numericality_of :order_max, :greater_than => 1
  end
  validates_presence_of :x
  validates_format_of :x, :with => /\A[a-zA-Z](?:,\s*[a-zA-Z])*\z/
  validates_presence_of :factor_min
  validates_presence_of :factor_max
  with_options :only_integer => true, :allow_blank => true do |problem|
    problem.validates_numericality_of :factor_min, :greater_than_or_equal_to => 0
    problem.validates_numericality_of :factor_max, :greater_than_or_equal_to => 0
  end

  def options_hash
    {
      :order_min    => order_min,
      :order_max    => order_max,
      :x            => split_x(x),
      :factor_minus => factor_minus,
      :factor_min   => factor_min,
      :factor_max   => factor_max
    }
  end

  def set_options(options)
    hash = ActiveSupport::JSON.decode(options)
    self.order_min    = hash["order_min"]
    self.order_max    = hash["order_max"]
    self.x            = hash["x"]
    self.factor_minus = hash["factor_minus"]
    self.factor_min   = hash["factor_min"]
    self.factor_max   = hash["factor_max"]
  end

  def validate
    check_min_max(:order_min, :order_max)
    check_min_max(:factor_min, :factor_max)
  end

end
