# == Schema Information
# Schema version: 20100309071953
#
# Table name: active_forms
#
#  unit_id         :
#  level           :integer
#  amount          :integer
#  lock_version    :integer
#  operators_plus  :boolean
#  operators_minus :boolean
#  operators_times :boolean
#  operators_div   :boolean
#  term_number     :integer
#  minus           :boolean
#  numerator_min   :integer
#  numerator_max   :integer
#  denominator_min :integer
#  denominator_max :integer
#

class FractionalArithmeticEditForm < ActiveForm
  acts_as_problem :operators => true, :term_number => true

  column :minus,           :type => :boolean
  column :numerator_min,   :type => :integer
  column :numerator_max,   :type => :integer
  column :denominator_min, :type => :integer
  column :denominator_max, :type => :integer

  validates_presence_of :numerator_min
  validates_presence_of :numerator_max
  validates_presence_of :denominator_min
  validates_presence_of :denominator_max
  with_options :only_integer => true, :allow_blank => true do |problem|
    problem.validates_numericality_of :numerator_min, :greater_than_or_equal_to => 1
    problem.validates_numericality_of :numerator_max, :greater_than_or_equal_to => 1
    problem.validates_numericality_of :denominator_min, :greater_than_or_equal_to => 2
    problem.validates_numericality_of :denominator_max, :greater_than_or_equal_to => 2
  end

  def options_hash
    {
      :minus           => minus,
      :numerator_min   => numerator_min,
      :numerator_max   => numerator_max,
      :denominator_min => denominator_min,
      :denominator_max => denominator_max,
      :term_number     => term_number,
      :operators       => operators,
    }
  end

  def set_options(options)
    hash = ActiveSupport::JSON.decode(options)
    self.minus = hash["minus"]
    self.numerator_min   = hash["numerator_min"]
    self.numerator_max   = hash["numerator_max"]
    self.denominator_min = hash["denominator_min"]
    self.denominator_max = hash["denominator_max"]
    self.term_number = hash["term_number"] if hash["term_number"]
    self.operators = hash["operators"] if hash["operators"]
  end

  def validate
    check_min_max(:numerator_min, :numerator_max)
    check_min_max(:denominator_min, :denominator_max)
    check_operators()
  end
end
