# == Schema Information
# Schema version: 20100309071953
#
# Table name: active_forms
#
#  unit_id              :
#  level                :integer
#  amount               :integer
#  lock_version         :integer
#  x                    :
#  numerator_term_min   :integer
#  numerator_term_max   :integer
#  denominator_term_min :integer
#  denominator_term_max :integer
#  factor_minus         :boolean
#  factor_min           :integer
#  factor_max           :integer
#

class FractionalExpressionArithmeticEditForm < ActiveForm
  acts_as_problem

  column :x
  column :numerator_term_min, :type => :integer
  column :numerator_term_max, :type => :integer
  column :denominator_term_min, :type => :integer
  column :denominator_term_max, :type => :integer
  column :factor_minus, :type => :boolean
  column :factor_min, :type => :integer
  column :factor_max, :type => :integer

  validates_presence_of :x
  validates_format_of   :x, :with => /\A[a-zA-Z](?:,\s*[a-zA-Z])*\z/
  validates_presence_of :numerator_term_min
  validates_presence_of :numerator_term_max
  validates_presence_of :denominator_term_min
  validates_presence_of :denominator_term_max
  validates_presence_of :factor_min
  validates_presence_of :factor_max

  with_options :only_integer => true, :allow_blank => true do |problem|
    problem.validates_numericality_of :numerator_term_min, :greater_than_or_equal_to => 1
    problem.validates_numericality_of :numerator_term_max, :greater_than_or_equal_to => 1
    problem.validates_numericality_of :denominator_term_min, :greater_than_or_equal_to => 1
    problem.validates_numericality_of :denominator_term_max, :greater_than_or_equal_to => 1
    problem.validates_numericality_of :factor_min, :greater_than_or_equal_to => 0
    problem.validates_numericality_of :factor_max, :greater_than_or_equal_to => 0
  end

  def options_hash
    {
      :x                    => split_x(x),
      :numerator_term_min   => numerator_term_min,
      :numerator_term_max   => numerator_term_max,
      :denominator_term_min => denominator_term_min,
      :denominator_term_max => denominator_term_max,
      :factor_minus         => factor_minus,
      :factor_min           => factor_min,
      :factor_max           => factor_max
    }
  end

  def set_options(options)
    hash = ActiveSupport::JSON.decode(options)
    self.x                    = hash["x"]
    self.numerator_term_min   = hash["numerator_term_min"]
    self.numerator_term_max   = hash["numerator_term_max"]
    self.denominator_term_min = hash["denominator_term_min"]
    self.denominator_term_max = hash["denominator_term_max"]
    self.factor_minus         = hash["factor_minus"]
    self.factor_min           = hash["factor_min"]
    self.factor_max           = hash["factor_max"]
  end

  def validate
    check_min_max(:numerator_term_min, :numerator_term_max)
    check_min_max(:denominator_term_min, :denominator_term_max)
    check_min_max(:factor_min, :factor_max)
  end
end
