# -*- coding: utf-8 -*-
# == Schema Information
# Schema version: 20100309071953
#
# Table name: learning_results
#
#  id                 :integer       not null, primary key
#  problem_id         :integer       not null
#  report_id          :integer       not null
#  problem_expression :text
#  answer             :text
#  example_solution   :text
#  correct            :boolean
#  created_at         :datetime
#  updated_at         :datetime
#  lock_version       :integer
#

class LearningResult < ActiveRecord::Base

  belongs_to :problem
  belongs_to :report

  validates_presence_of :problem_id
  validates_presence_of :report_id

  before_save :verify_answer

  def example_solution_as_ascii_math_ml
    return example_solution if problem.unit.symbol == "quadratic_equation"
    Mint::Builder.build(self.example_solution).to_ascii_math_ml
  end

  private

  def verify_answer
    return verify_answer_with_quadratic_equation if problem.unit.symbol == "quadratic_equation"
    solver = self.problem.solver
    example_solution_expression = case problem.unit.symbol
                                  when "linear_function_graph"
                                    hash = ActiveSupport::JSON.decode(problem_expression)
                                    solver.solve(hash.symbolize_keys)
                                  else
                                    solver.solve(Mint::Builder.build(problem_expression))
                                  end
    answer_expression = Mint::Builder.build(self.answer)
    self.example_solution = example_solution_expression.normalize
    self.correct = answer_expression == example_solution_expression
    true
  end

  def verify_answer_with_quadratic_equation
    solver = self.problem.solver
    example_solution_expressions = solver.solve(Mint::Builder.build(problem_expression))
    if example_solution_expressions.size == 1
      example_solution_expressions += example_solution_expressions
    end
    answer_expressions = self.answer.scan(%r!\((.+?), *(.+?)\)!).flatten.map{|str|
      Mint::Builder.build(str)
    }
    self.example_solution = '(%s, %s)' % example_solution_expressions.map(&:normalize)
    # 答えの順番が違う場合は誤答とする
    self.correct = answer_expressions == example_solution_expressions
    true
  end

end

