# == Schema Information
# Schema version: 20100309071953
#
# Table name: active_forms
#
#  unit_id         :
#  level           :integer
#  amount          :integer
#  lock_version    :integer
#  minus           :boolean
#  x_minus         :boolean
#  x_min           :integer
#  x_max           :integer
#  numerator_min   :integer
#  numerator_max   :integer
#  denominator_min :integer
#  denominator_max :integer
#

class LinearFunctionGraphEditForm < ActiveForm
  acts_as_problem

  column :minus, :type => :boolean
  column :x_minus, :type => :boolean
  column :x_min,   :type => :integer
  column :x_max,   :type => :integer
  column :numerator_min,   :type => :integer
  column :numerator_max,   :type => :integer
  column :denominator_min,   :type => :integer
  column :denominator_max,   :type => :integer

  validates_presence_of :x_min
  validates_presence_of :x_max
  validates_presence_of :numerator_min
  validates_presence_of :numerator_max
  validates_presence_of :denominator_min
  validates_presence_of :denominator_max

  with_options :only_integer => true, :allow_blank => true do |problem|
    problem.validates_numericality_of :x_min, :greater_than_or_equal_to => 0
    problem.validates_numericality_of :x_max, :greater_than_or_equal_to => 0
    problem.validates_numericality_of :numerator_min, :greater_than_or_equal_to => 1
    problem.validates_numericality_of :numerator_max, :greater_than_or_equal_to => 1
    problem.validates_numericality_of :denominator_min, :greater_than_or_equal_to => 1
    problem.validates_numericality_of :denominator_max, :greater_than_or_equal_to => 1
  end

  def options_hash
    {
      :minus           => minus,
      :x_minus         => x_minus,
      :x_min           => x_min,
      :x_max           => x_max,
      :numerator_min   => numerator_min,
      :numerator_max   => numerator_max,
      :denominator_min => denominator_min,
      :denominator_max => denominator_max,
    }
  end

  def set_options(options)
    hash = ActiveSupport::JSON.decode(options)
    self.minus           = hash["minus"]
    self.x_minus         = hash["x_minus"]
    self.x_min           = hash["x_min"]
    self.x_max           = hash["x_max"]
    self.numerator_min   = hash["numerator_min"]
    self.numerator_max   = hash["numerator_max"]
    self.denominator_min = hash["denominator_min"]
    self.denominator_max = hash["denominator_max"]
  end

  def validate
    check_min_max(:x_min, :x_max)
    check_min_max(:numerator_min, :numerator_max)
    check_min_max(:denominator_min, :denominator_max)
  end

end

