# == Schema Information
# Schema version: 20100309071953
#
# Table name: active_forms
#
#  unit_id         :
#  level           :integer
#  amount          :integer
#  lock_version    :integer
#  operators_plus  :boolean
#  operators_minus :boolean
#  operators_times :boolean
#  operators_div   :boolean
#  term_number     :integer
#  minus           :boolean
#  min             :integer
#  max             :integer
#

class OrdinaryArithmeticEditForm < ActiveForm
  acts_as_problem :operators => true, :term_number => true

  column :minus, :type => :boolean
  column :min,   :type => :integer
  column :max,   :type => :integer

  validates_presence_of :min
  validates_presence_of :max

  with_options :only_integer => true, :allow_blank => true do |problem|
    problem.validates_numericality_of :min, :greater_than_or_equal_to => 0
    problem.validates_numericality_of :max, :greater_than_or_equal_to => 0
  end

  def options_hash
    {
      :minus => minus,
      :min   => min,
      :max   => max,
      :term_number => term_number,
      :operators => operators,
    }
  end

  def set_options(options)
    hash = ActiveSupport::JSON.decode(options)
    self.minus = hash["minus"]
    self.min   = hash["min"]
    self.max   = hash["max"]
    self.term_number = hash["term_number"] if hash["term_number"]
    self.operators = hash["operators"] if hash["operators"]
  end

  def validate
    check_min_max(:min, :max)
    check_operators()
  end
end
