# == Schema Information
# Schema version: 20100309071953
#
# Table name: active_forms
#
#  unit_id           :
#  level             :integer
#  amount            :integer
#  lock_version      :integer
#  order_min         :integer
#  order_max         :integer
#  x                 :
#  factor_minus      :boolean
#  factor_min        :integer
#  factor_max        :integer
#  coefficient_minus :boolean
#  coefficient_min   :integer
#  coefficient_max   :integer
#  numerator_minus   :boolean
#  numerator_min     :integer
#  numerator_max     :integer
#

class PartialFractionExpansionEditForm < ActiveForm
  acts_as_problem

  column :order_min, :type => :integer
  column :order_max, :type => :integer
  column :x
  column :factor_minus, :type => :boolean
  column :factor_min,   :type => :integer
  column :factor_max,   :type => :integer
  column :coefficient_minus, :type => :boolean
  column :coefficient_min,   :type => :integer
  column :coefficient_max,   :type => :integer
  column :numerator_minus, :type => :boolean
  column :numerator_min,   :type => :integer
  column :numerator_max,   :type => :integer

  validates_presence_of :order_min
  validates_presence_of :order_max
  with_options :only_integer => true, :allow_blank => true do |problem|
    problem.validates_numericality_of :order_min, :greater_than => 1
    problem.validates_numericality_of :order_max, :greater_than => 1
  end
  validates_presence_of :x
  validates_format_of :x, :with => /\A[a-zA-Z](?:,\s*[a-zA-Z])*\z/
  validates_presence_of :factor_min
  validates_presence_of :factor_max
  with_options :only_integer => true, :allow_blank => true do |problem|
    problem.validates_numericality_of :factor_min, :greater_than_or_equal_to => 0
    problem.validates_numericality_of :factor_max, :greater_than_or_equal_to => 0
  end
  validates_presence_of :coefficient_min
  validates_presence_of :coefficient_max
  with_options :only_integer => true, :allow_blank => true do |problem|
    problem.validates_numericality_of :coefficient_min, :greater_than_or_equal_to => 1
    problem.validates_numericality_of :coefficient_max, :greater_than_or_equal_to => 1
  end
  validates_presence_of :numerator_min
  validates_presence_of :numerator_max
  with_options :only_integer => true, :allow_blank => true do |problem|
    problem.validates_numericality_of :numerator_min, :greater_than_or_equal_to => 1
    problem.validates_numericality_of :numerator_max, :greater_than_or_equal_to => 1
  end

  def options_hash
    {
      :order_min         => order_min,
      :order_max         => order_max,
      :x                 => split_x(x),
      :factor_minus      => factor_minus,
      :factor_min        => factor_min,
      :factor_max        => factor_max,
      :coefficient_minus => coefficient_minus,
      :coefficient_min   => coefficient_min,
      :coefficient_max   => coefficient_max,
      :numerator_minus   => numerator_minus,
      :numerator_min     => numerator_min,
      :numerator_max     => numerator_max,
    }
  end

  def set_options(options)
    hash = ActiveSupport::JSON.decode(options)
    self.order_min         = hash["order_min"]
    self.order_max         = hash["order_max"]
    self.x                 = hash["x"]
    self.factor_minus      = hash["factor_minus"]
    self.factor_min        = hash["factor_min"]
    self.factor_max        = hash["factor_max"]
    self.coefficient_minus = hash["coefficient_minus"]
    self.coefficient_min   = hash["coefficient_min"]
    self.coefficient_max   = hash["coefficient_max"]
    self.numerator_minus   = hash["numerator_minus"]
    self.numerator_min     = hash["numerator_min"]
    self.numerator_max     = hash["numerator_max"]
  end

  def validate
    check_min_max(:order_min, :order_max)
    check_min_max(:factor_min, :factor_max)
    check_min_max(:coefficient_min, :coefficient_max)
    check_min_max(:numerator_min, :numerator_max)
  end

end
