# == Schema Information
# Schema version: 20100309071953
#
# Table name: problems
#
#  id                       :integer       not null, primary key
#  unit_id                  :integer       not null
#  problem_group_problem_id :integer
#  level                    :integer       default(0), not null
#  amount                   :integer       default(1), not null
#  options                  :text          not null
#  lock_version             :integer       default(0), not null
#  created_at               :datetime
#  updated_at               :datetime
#

class Problem < ActiveRecord::Base

  belongs_to :unit

  has_many :learning_results
  has_many :problem_groups_problems, :class_name => 'ProblemGroupsProblems', :dependent => :destroy
  has_many :problem_groups, :through => :problem_groups_problems

  validates_presence_of :unit_id

  validates_presence_of  :level
  validates_inclusion_of :level, :in => 0..100

  validates_presence_of :amount
  validates_inclusion_of :amount, :in => 1..100

  validates_presence_of :options


  def label
    "#{unit.name} Lv.#{level}"
  end

  def generate
    o = ActiveSupport::JSON.decode(options).symbolize_keys
    o.each{|k, v| v.symbolize_keys! if Hash === v }
    unit.generator(amount).generate(o).map{|v|
      case v
      when String then Mint::Builder.build(v)
      else v
      end
    }
  end

  def solver
    unit.solver
  end

end
