# == Schema Information
# Schema version: 20100309071953
#
# Table name: active_forms
#
#  unit_id      :
#  level        :integer
#  amount       :integer
#  lock_version :integer
#  answer_type  :
#  r_max        :integer
#  p_max        :integer
#  q_max        :integer
#

class QuadraticEquationEditForm < ActiveForm
  acts_as_problem

  column :answer_type
  column :r_min, :type => :integer
  column :r_max, :type => :integer
  column :p_minus, :type => :boolean
  column :p_min, :type => :integer
  column :p_max, :type => :integer
  column :q_min, :type => :integer
  column :q_max, :type => :integer

  validates_presence_of :answer_type
  validates_inclusion_of :answer_type, :in => %w[rational irrational imaginary]

  validates_presence_of :r_min
  validates_presence_of :r_max
  validates_presence_of :p_min
  validates_presence_of :p_max
  validates_presence_of :q_min
  validates_presence_of :q_max

  with_options :only_integer => true, :allow_blank => true do |problem|
    problem.validates_numericality_of :r_min, :greater_than_or_equal_to => 1
    problem.validates_numericality_of :r_max, :greater_than_or_equal_to => 1
    problem.validates_numericality_of :p_min, :greater_than_or_equal_to => 0
    problem.validates_numericality_of :p_max, :greater_than_or_equal_to => 0
    problem.validates_numericality_of :q_min, :greater_than_or_equal_to => 0
    problem.validates_numericality_of :q_max, :greater_than_or_equal_to => 0
  end

  def options_hash
    {
      :answer_type => answer_type,
      :r_min       => r_min,
      :r_max       => r_max,
      :p_minus     => p_minus,
      :p_min       => p_min,
      :p_max       => p_max,
      :q_min       => q_min,
      :q_max       => q_max,
    }
  end

  def set_options(options)
    hash = ActiveSupport::JSON.decode(options)
    self.answer_type = hash["answer_type"]
    self.r_min       = hash["r_min"]
    self.r_max       = hash["r_max"]
    self.p_minus     = hash["p_minus"]
    self.p_min       = hash["p_min"]
    self.p_max       = hash["p_max"]
    self.q_min       = hash["q_min"]
    self.q_max       = hash["q_max"]
  end

  def validate
    check_min_max(:p_min, :p_max)
    check_min_max(:q_min, :q_max)
    check_min_max(:r_min, :r_max)
  end
end
