# == Schema Information
# Schema version: 20100309071953
#
# Table name: reports
#
#  id               :integer       not null, primary key
#  user_id          :integer       not null
#  problem_group_id :integer       not null
#  submitted        :boolean       not null
#  created_at       :datetime
#  updated_at       :datetime
#

class Report < ActiveRecord::Base
  belongs_to :user
  belongs_to :problem_group
  has_many :learning_results

  validates_presence_of :user_id
  validates_presence_of :problem_group_id

  named_scope :by_user, lambda{|user_id|
    { :conditions => { :user_id => user_id } }
  }
  named_scope :by_date, lambda{|date|
    { :conditions => [%Q! created_at between ? and ? !,
                      date.to_time.midnight, (date+1).to_time.midnight] }
  }
  named_scope :by_range, lambda{|start_date, end_date|
    { :conditions => [%Q! created_at between ? and ? !,
                     start_date.to_time.midnight, (end_date+1).to_time.midnight] }
  }
  named_scope :by_problem_group, lambda{|problem_group_id|
    { :conditions => { :problem_group_id => problem_group_id } }
  }

  def summary
    all = self.learning_results
    "#{all.select(&:correct?).size}/#{all.size}"
  end
end
