# == Schema Information
# Schema version: 20100309071953
#
# Table name: active_forms
#
#  unit_id         :
#  level           :integer
#  amount          :integer
#  lock_version    :integer
#  operators_plus  :boolean
#  operators_minus :boolean
#  term_number     :integer
#  min             :integer
#  max             :integer
#  single_term_min :integer
#  single_term_max :integer
#  use_coefficient :boolean
#  use_power       :boolean
#

class SquareRootArithmeticEditForm < ActiveForm
  acts_as_problem :operators => %w[plus minus], :term_number => true

  column :min, :type => :integer
  column :max, :type => :integer
  column :single_term_min, :type => :integer
  column :single_term_max, :type => :integer
  column :use_coefficient, :type => :boolean
  column :use_power, :type => :boolean

  validates_presence_of :min
  validates_presence_of :max
  validates_presence_of :single_term_min
  validates_presence_of :single_term_max

  with_options :only_integer => true, :allow_blank => true do |problem|
    problem.validates_numericality_of :min, :greater_than_or_equal_to => 1
    problem.validates_numericality_of :max, :greater_than_or_equal_to => 1
    problem.validates_numericality_of :single_term_min, :greater_than_or_equal_to => 1
    problem.validates_numericality_of :single_term_max, :greater_than_or_equal_to => 1
  end

  def options_hash
    {
      :min             => min,
      :max             => max,
      :single_term_min => single_term_min,
      :single_term_max => single_term_max,
      :term_number     => term_number,
      :operators       => operators,
      :use_coefficient => use_coefficient,
      :use_power       => use_power,
    }
  end

  def set_options(options)
    hash = ActiveSupport::JSON.decode(options)
    self.min             = hash["min"]
    self.max             = hash["max"]
    self.single_term_min = hash["single_term_min"]
    self.single_term_max = hash["single_term_max"]
    self.term_number     = hash["term_number"] if hash["term_number"]
    self.operators       = hash["operators"] if hash["operators"]
  end

  def validate
    check_min_max(:min, :max)
    check_min_max(:single_term_min, :single_term_max)
    check_operators()
  end
end
