# == Schema Information
# Schema version: 20100309071953
#
# Table name: units
#
#  id              :integer       not null, primary key
#  name            :string(40)    not null
#  symbol          :string(40)    not null, index_units_on_symbol
#  form_class_name :string(255)
#  position        :integer
#  created_at      :datetime
#  updated_at      :datetime
#

class Unit < ActiveRecord::Base

  has_many :problems

  validates_presence_of :name
  validates_length_of :name, :maximum => 40

  validates_presence_of   :symbol
  validates_length_of     :symbol, :maximum => 40, :allow_nil => true
  validates_format_of     :symbol, :with => /\A[a-z_]+\z/
  validates_uniqueness_of :symbol

  acts_as_list

  def form_class
    if form_class_name && !form_class_name.blank?
      form_class_name.constantize
    else
      "#{symbol}_edit_form".classify.constantize
    end
  end

  def form_template
    "#{symbol}_edit_form"
  end

  def generator(amount)
    Mint::Generator::Factory.create(symbol, amount)
  end

  def solver
    case symbol
    when "linear_function_graph"
      Mint::Solver::Factory.create(symbol, Mint::Solver::Simple)
    else
      Mint::Solver::Factory.create(symbol)
    end
  end
end
