# == Schema Information
# Schema version: 20100309071953
#
# Table name: users
#
#  id                   :integer       not null, primary key
#  affiliation_id       :integer
#  matriculation_number :string(100)
#  name                 :string(200)
#  email                :string(200)
#  admission_date       :date
#  created_at           :datetime
#  updated_at           :datetime
#  lock_version         :integer       default(0), not null
#

class User < ActiveRecord::Base

  ROLES = %w[admin teacher]

  multi_auth

  has_many :reports
  belongs_to :affiliation

  acts_as_authorization_subject

  validates_uniqueness_of :matriculation_number, :scope => :affiliation_id, :allow_blank => true
  validates_length_of :name, :maximum => 200, :allow_nil => true
  validates_email_format_of :email, :allow_nil => true, :allow_blank => true
  validates_uniqueness_of :email, :allow_nil => true, :allow_blank => true

  def role
    ROLES.detect{|r| self.has_role?(r) }
  end

  def before_create
    self.admission_date = Date.today
  end

  def have_profile?
     not [affiliation_id, matriculation_number, name, email].any?(&:blank?)
  end
end
