# -*- coding: utf-8 -*-
# This file should contain all the record creation needed to seed the database with its default values.
# The data can then be loaded with the rake db:seed (or created alongside the db with db:setup).
#
# Examples:
#   
#   cities = City.create([{ :name => 'Chicago' }, { :name => 'Copenhagen' }])
#   Major.create(:name => 'Daley', :city => cities.first)

ActiveRecord::Base.transaction do
  admin = User.create!(:name => "admin",
                       :email => "admin@example.com",
                       :admission_date => Date.today)
  hashed_password = EmailCredential.create_hashed_password("monkey")
  e = admin.email_credentials.build(:email            => admin.email,
                                    :hashed_password  => hashed_password,
                                    :activation_token => "1"*20,
                                    :activated_at     => Time.now)
  e.save!
  admin.has_role!(:admin)

  user = User.create!(:name => "user",
                      :email => "user@example.com",
                      :admission_date => Date.today)
  hashed_password = EmailCredential.create_hashed_password("monkey")
  e = user.email_credentials.build(:email => user.email,
                                   :hashed_password  => hashed_password,
                                   :activation_token => "2"*20,
                                   :activated_at     => Time.now)
  e.save!

  teacher = User.create!(:name => "teacher", :email => "teacher@example.com")
  hashed_password = EmailCredential.create_hashed_password("monkey")
  e = teacher.email_credentials.build(:email => teacher.email,
                                      :hashed_password  => hashed_password,
                                      :activation_token => "3"*20,
                                      :activated_at     => Time.now)
  e.save!
  teacher.has_role!(:teacher)

  doc_option_ordinary_arithmetic = <<-EOD.gsub(/^ {4}/, '')
    ! 整数の四則演算を生成するジェネレータ

    !! オプション

    !!! term_number
    生成する式の項数を１以上の整数で指定します。

    !!! operators
    使用可能な演算子を指定します。
    +, -, *, div の４種類から使用したいものを配列で指定します。

    !!! min
    生成する式の各項の最小値を１以上の整数で指定します。

    !!! max
    生成する式の各項の最大値を１以上の整数で指定します。
    ''min'' よりも小さい値を指定することは出来ません。

    !!! minus
    真を指定すると、負の値も生成します。
  EOD
  doc_option_decimal_arithmetic = <<-EOD.gsub(/^ {4}/, '')
    ! 小数の四則演算を生成するジェネレータ

    !! オプション

    !!! term_number
    生成する式の項数を１以上の整数で指定します。
    !!! operators
    使用可能な演算子を指定します。
    +, -, *, div の４種類から使用したいものを配列で指定します。
    !!! min
    生成する式の各項の最小値を０以上の整数で指定します。
    !!! max
    生成する式の各項の最大値を０以上の整数で指定します。
    ''min'' よりも小さい値を指定することは出来ません。
    !!! minus
    真を指定すると、負も値を生成します。
    !!! digits
    小数第何位までの小数を生成するかを１以上の整数で指定します。
  EOD
  doc_option_fractional_arithmetic = <<-EOD.gsub(/^ {4}/, '')
    ! 分数の四則演算を生成するジェネレータ

    !! オプション

    !!! term_number
    生成する式の項数を１以上の整数で指定します。
    !!! operators
    使用可能な演算子を指定します。
    +, -, *, div の４種類から使用したいものを配列で指定します。
    !!! minus
    真を指定すると、負の値も生成します。
    !!! numerator_min
    生成する分子の最小値を１以上の整数で指定します。
    !!! numerator_max
    生成する分子の最大値を１以上の整数で指定します。
    ''numerator_min'' より小さい値を指定することは出来ません。
    !!! denominator_min
    生成する分母の最小値を１以上の整数で指定します。
    !!! denominator_max
    生成する分母の最大値を１以上の整数で指定します。
    ''denominator_min'' より小さい値を指定することは出来ません。
  EOD
  doc_option_square_root_arithmetic = <<-EOD.gsub(/^ {4}/, '')
    ! 無理数の四則演算を生成するジェネレータ

    !! オプション

    !!! term_number
    生成する式の項数を１以上の整数で指定します。
    !!! operators
    使用可能な演算子を指定します。
    +, - の２種類から使用したいものを配列で指定します。
    !!! min
    生成する式の各項の最小値を０以上の整数で指定します。
    !!! max
    生成する式の各項の最大値を０以上の整数で指定します。
    ''min'' よりも小さい値を指定することは出来ません。
    !!! upper_limit
    すべてが数字だった場合は強制的に１つ根号を追加します。
    そのときの根号内の数字の最大値を2以上の整数で指定します。
    なお、最小値は自動的に２に設定されます。
    !!! single_term_min
    生成する式の項数の最小値を１以上の整数で指定します。
    !!! single_term_max
    生成する式の項数の最大値を１以上の整数で指定します。
    ''single_term_min'' より小さい値は指定出来ません。　
    !!! use_coefficient
    真を指定すると根号を含む項の係数を生成します。
    !!! use_power
    真を指定すると２乗項を生成します。
  EOD
  doc_option_complex_number_arithmetic = <<-EOD.gsub(/^ {4}/, '')
    ! 複素数の問題を生成するジェネレータ

    !! オプション

    !!! term_number
    生成する式の項数を１以上の整数で指定します。
    !!! operators
    使用可能な演算子を指定します。
    *, / の２種類から使用したいものを配列で指定します。
    !!! min
    生成する式の各項の最小値を０以上の整数で指定します。
    !!! max
    生成する式の各項の最大値を０以上の整数で指定します。
    ''min'' よりも小さい値を指定することは出来ません。
    !!! fractional_mode
    真を指定すると分数形式の項を生成します。
  EOD
  doc_option_complex_fractional_arithmetic = <<-EOD.gsub(/^ {4}/, '')
    ! 繁分数の問題を生成するジェネレータ

    !! オプション

    !!! term_number
    生成する式の項数を１以上の整数で指定します。
    !!! operators
    使用可能な演算子を指定します。
    +, -, * の３種類から使用したいものを配列で指定します。
    !!! numerator_term_min
    生成する分母の項数の最小値を１以上の整数で指定します。
    !!! numerator_term_max
    生成する分母の項数の最大値を１以上の整数で指定します。
    ''numerator_term_min'' より小さい値を指定することはできません。
    !!! denominator_term_min
    生成する分子の項数の最小値を１以上の整数で指定します。
    !!! denominator_term_max
    生成する分子の項数の最大値を１以上の整数で指定します。
    ''denominator_term_min'' より小さい値を指定することはできません。
  EOD
  doc_option_expansion = <<-EOD.gsub(/^ {4}/, '')
    ! 式展開の問題を生成するジェネレータ

    !! オプション

    !!! order_min
    生成する方程式の次数の最小値を１以上の整数で指定します。
    !!! order_max
    生成する方程式の次数の最大値を１以上の整数で指定します。
    ''order_min'' より小さい値を指定することは出来ません。
    !!! x
    使用可能な変数の文字を配列で指定します。
    文字列の代わりに配列の形で文字列のペアを
    指定すると、２変数を使用した式を生成します。
    !!! factor_min
    生成する式の各項の最小値を１以上の整数で指定します。
    !!! factor_max
    生成する式の各項の最大値を１以上の整数で指定します。
    ''factor_min'' よりも小さい値を指定することは出来ません。
    !!! factor_minus
    真を指定すると、負も値を生成します。
    !!! coefficient_min
    係数の最小値を１以上の整数で指定します。
    !!! coefficient_max
    係数の最大値を１以上の整数で指定します。
    ''coefficient_min'' より小さい値は指定できません。
    !!! coefficient_minus
    真を指定すると分母の係数として負の値を生成します。
  EOD
  doc_option_factorization = <<-EOD.gsub(/^ {4}/, '')
    ! 因数分解の問題を生成するジェネレータ

    !! オプション

    !!! order_min
    生成する方程式の次数の最小値を２もしくは３を指定します。
    !!! order_max
    生成する方程式の次数の最大値を２もしくは３を指定します。
    ''order_min'' より小さい値を指定することは出来ません。
    !!! x
    使用可能な変数の文字を配列で指定します。
    文字列の代わりに配列の形で文字列のペアを
    指定すると、２変数を使用した式を生成します。
    !!! factor_min
    生成する式の各項の最小値を０以上の整数で指定します。
    !!! factor_max
    生成する式の各項の最大値を０以上の整数で指定します。
    ''factor_min'' よりも小さい値を指定することは出来ません。
    !!! factor_minus
    真を指定すると、負も値を生成します。
  EOD
  doc_option_fractional_expression_arithmetic = <<-EOD.gsub(/^ {4}/, '')
    ! 分数式の問題を生成するジェネレータ

    !! オプション

    !!! term_number
    生成する式の項数を１以上の整数で指定します。
    !!! operators
    使用可能な演算子を指定します。
    +, -, *, div の４種類から使用したいものを配列で指定します。
    !!! numerator_term_min
    生成する分子の項数の最小値を１以上の整数で指定します。
    !!! numerator_term_max
    生成する分子の項数の最大値を１以上の整数で指定します。
    ''numerator_term_min'' より小さい値を指定することは出来ません。
    !!! denominator_term_min
    生成する分母の項数の最小値を１以上の整数で指定します。
    !!! denominator_term_max
    生成する分母の項数の最大値を１以上の整数で指定します。
    ''denominator_term_min'' より小さい値を指定することは出来ません。
    !!! factor_min
    生成する式の各項の最小値を０以上の整数で指定します。
    !!! factor_max
    生成する式の各項の最大値を０以上の整数で指定します。
    ''factor_min'' よりも小さい値を指定することは出来ません。
    !!! factor_minus
    真を指定すると、負の値も生成します。
  EOD
  doc_option_partial_fraction_expansion = <<-EOD.gsub(/^ {4}/, '')
    ! 部分分数展開の問題を生成するジェネレータ

    !! オプション

    !!! order_min
    生成する式の次数の最小値を１以上の整数で指定します。
    !!! order_max
    生成する式の次数の最大値を１以上の整数で指定します。
    ''order_min'' より小さい値を指定することは出来ません。
    !!! x
    使用可能な変数の文字を配列で指定します。
    文字列の代わりに配列の形で文字列のペアを
    指定すると、２変数を使用した式を生成します。
    !!! factor_min
    生成する式の各項の最小値を０以上の整数で指定します。
    !!! factor_max
    生成する式の各項の最大値を０以上の整数で指定します。
    ''factor_min'' よりも小さい値を指定することは出来ません。
    !!! factor_minus
    真を指定すると、負も値を生成します。
    !!! coefficient_min
    分母の係数の最小値を１以上の整数で指定します。
    !!! coefficient_max
    分母の係数の最大値を１以上の整数で指定します。
    ''coefficient_min'' より小さい値は指定できません。
    !!! coefficient_minus
    真を指定すると分母の係数として負の値を生成します。
    !!! numerator_min
    生成する分子の最小値を０以上の整数で指定します。　
    !!! numerator_max
    生成する分子の最大値を０以上の整数で指定します。　
    ''numerator_min'' より小さい値を指定することは出来ません。
    !!! numerator_minus
    真を指定すると分子に負の値も生成します。
  EOD
  doc_option_linear_function_graph = <<-EOD.gsub(/^ {4}/, '')
    ! 一次関数の問題を生成するジェネレータ

    !! オプション

    !!! x_minus
    真を指定すると x の値として負の値も生成します。
    !!! x_min
    x の最小値を１以上の整数で指定します。
    !!! x_max
    x の最大値を１以上の整数で指定します。
    ''x_min'' より小さい値を指定することは出来ません。
    !!! minus
    真を指定すると傾きや切片として負の値も生成します。
    !!! numerator_min
    傾きと切片で使用する分子の最小値を１以上の整数で指定します。
    !!! numerator_max
    傾きと切片で使用する分子の最大値を１以上の整数で指定します。
    ''numerator_min'' より小さい値を指定することは出来ません。
    !!! denominator_min
    傾きと切片で使用する分母の最小値を１以上の整数で指定します。
    !!! denominator_max
    傾きと切片で使用する分母の最大値を１以上の整数で指定します。
    ''denominator_min'' より小さい値を指定することは出来ません。
  EOD
  doc_option_quadratic_equation = <<-EOD.gsub(/^ {4}/, '')
    ! 二次方程式の問題を生成するジェネレータ

    !! オプション

    !!! answer_type
    解の種類を指定します。
    以下の３種類から１つ指定します。
    :rational: 整数解を生成します。
    :irrational: 無理数解を生成します。
    :imaginary: 虚数解を生成します。
    !!! r_min
    r の最小値を１以上の整数で指定します。
    !!! r_max
    r の最大値を１以上の整数で指定します。
    素数を指定することは出来ません。
    !!! p_min
    p の最小値を０以上の整数でしていします。
    !!! p_max
    p の最大値を１以上の整数で指定します。
    素数を指定することは出来ません。
    !!! p_minus
    真を指定すると、p の値として負の値も生成します。
    !!! q_min
    q の最小値を０以上の整数で指定します。
    !!! q_max
    q の最大値を１以上の整数で指定します。
    素数を指定することは出来ません。
    また平方数の数列に属する値を指定することも出来ません。

    !! r, p, q について

    r, p, q を用いて 係数 a, b, c を生成します.
    それぞれ以下の式で求めることができます.

    * a' = r^2
    * b' = -2pr
    * c' = (p^2 - q)
    * gcd = GCD(a', b', c')

    とすると

    * a = a' / gcd
    * b = b' / gcd
    * c = c' / gcd
  EOD

  [
   ["ordinary_arithmetic"              , "整数の四則演算",      doc_option_ordinary_arithmetic],
   ["decimal_arithmetic"               , "小数の四則演算",      doc_option_decimal_arithmetic],
   ["fractional_arithmetic"            , "分数の四則演算",      doc_option_fractional_arithmetic],
   ["square_root_arithmetic"           , "根号を含む数の計算",  doc_option_fractional_arithmetic],
   ["complex_number_arithmetic"        , "複素数の計算",        doc_option_square_root_arithmetic],
   ["complex_fractional_arithmetic"    , "繁分数の計算",        doc_option_complex_fractional_arithmetic],
   ["expansion"                        , "式の展開",            doc_option_expansion],
   ["factorization"                    , "因数分解",            doc_option_factorization],
   ["fractional_expression_arithmetic" , "分数式の計算",        doc_option_fractional_expression_arithmetic],
   ["partial_fraction_expansion"       , "部分分数の展開",      doc_option_partial_fraction_expansion],
   ["linear_function_graph"            , "一次関数のグラフ",    doc_option_linear_function_graph],
   ["quadratic_equation"               , "二次方程式",          doc_option_quadratic_equation],
  ].each do |symbol, name, explanation|
    Unit.create!(:symbol => symbol, :name => name, :explanation => explanation)
  end

  problem_group = ProblemGroup.new(:label => "整数の四則演算",
                                   :description => "与えられた式を計算してください。")
  problem_group.problems.build(:unit => Unit.find_by_symbol("ordinary_arithmetic"), :level => 0,
                               :amount => 2,
                               :options => {
                                 :minus => true, :min => 0, :max => 10,
                                 :term_number => 2,
                                 :operators => %w[+ - * div]
                               }.to_json)
  problem_group.problems.build(:unit => Unit.find_by_symbol("ordinary_arithmetic"), :level => 1,
                               :amount => 2,
                               :options => {
                                 :minus => true, :min => 0, :max => 100,
                                 :term_number => 2,
                                 :operators => %w[+ - * div]
                               }.to_json)
  problem_group.problems.build(:unit => Unit.find_by_symbol("ordinary_arithmetic"), :level => 2,
                               :amount => 2,
                               :options => {
                                 :minus => true, :min => 0, :max => 100,
                                 :term_number => 3,
                                 :operators => %w[+ - * div]
                               }.to_json)
  problem_group.save!

  problem_group = ProblemGroup.new(:label => "小数の四則演算",
                                   :description => "与えられた式を計算してください。")
  problem_group.problems.build(:unit => Unit.find_by_symbol("decimal_arithmetic"), :level => 0,
                               :amount => 2,
                               :options => {
                                 :minus => true, :digits => 2, :min => 0, :max => 10.0,
                                 :term_number => 2,
                                 :operators => %w[+ - * div]
                               }.to_json)
  problem_group.problems.build(:unit => Unit.find_by_symbol("decimal_arithmetic"), :level => 1,
                               :amount => 2,
                               :options => {
                                 :minus => true, :digits => 2, :min => 0, :max => 100.0,
                                 :term_number => 3,
                                 :operators => %w[+ - * div]
                               }.to_json)
  problem_group.save!

  problem_group = ProblemGroup.new(:label => "分数の四則演算",
                                   :description => "与えられた式を計算してください。答えは既約にしてください。")
  problem_group.problems.build(:unit => Unit.find_by_symbol("fractional_arithmetic"), :level => 0,
                               :amount => 2,
                               :options => {
                                 :minus => true,
                                 :numerator_min => 1, :numerator_max => 10,
                                 :denominator_min => 2, :denominator_max => 10
                               }.to_json)
  problem_group.problems.build(:unit => Unit.find_by_symbol("fractional_arithmetic"), :level => 1,
                               :amount => 2,
                               :options => {
                                 :minus => true,
                                 :numerator_min => 1, :numerator_max => 20,
                                 :denominator_min => 2, :denominator_max => 20
                               }.to_json)
  problem_group.save!

  problem_group = ProblemGroup.new(:label => "根号を含む数の計算",
                                   :description => "与えられた式を計算してください。答えは分母を有理化してください。")
  problem_group.problems.build(:unit => Unit.find_by_symbol("square_root_arithmetic"), :level => 0,
                               :amount => 2,
                               :options => {
                                 :term_number => 2, :operators => %w[+ -],
                                 :min => 1, :max => 50,
                                 :single_term_min => 1, :single_term_max => 2,
                                 :use_coefficient => true, :use_power => true,
                               }.to_json)
  problem_group.save!

  problem_group = ProblemGroup.new(:label => "複素数の計算",
                                   :description => "与えられた式を計算してください。答えは分母を有理化してください。")
  problem_group.problems.build(:unit => Unit.find_by_symbol("complex_number_arithmetic"), :level => 0,
                               :amount => 2,
                               :options => {
                                 :term_number => 2, :operators => %w[+ - * /],
                                 :min => 0, :max => 30,
                                 :fractional_mode => false,
                               }.to_json)
  problem_group.save!

  problem_group = ProblemGroup.new(:label => "繁分数の計算",
                                   :description => "与えられた式を簡単にしてください。")
  problem_group.problems.build(:unit => Unit.find_by_symbol("complex_fractional_arithmetic"), :level => 0,
                               :amount => 2,
                               :options => {
                                 :operators => %w[+ - *],
                                 :numerator_term_min => 1, :numerator_term_max => 2,
                                 :denominator_term_min => 2, :denominator_term_max => 2,
                               }.to_json)
  problem_group.save!

  problem_group = ProblemGroup.new(:label => "分数式の計算",
                                   :description => "与えられた式を簡単にしてください。")
  problem_group.problems.build(:unit => Unit.find_by_symbol("fractional_expression_arithmetic"), :level => 0,
                               :amount => 2,
                               :options => {
                                 :x => ['x'],
                                 :numerator_term_min   => 1, :numerator_term_max   => 1,
                                 :denominator_term_min => 1, :denominator_term_max => 1,
                                 :factor_minus => false, :factor_min => 1, :factor_max => 9,
                               }.to_json)
  problem_group.save!

  problem_group = ProblemGroup.new(:label => "部分分数の展開",
                                   :description => "与えられた式を簡単にしてください。")
  problem_group.problems.build(:unit => Unit.find_by_symbol("partial_fraction_expansion"), :level => 0,
                               :amount => 2,
                               :options => {
                                 :order_min         => 2,     :order_max       => 2, :x               => ['x'],
                                 :factor_minus      => false, :factor_min      => 1, :factor_max      => 100,
                                 :coefficient_minus => false, :coefficient_min => 1, :coefficient_max => 1,
                                 :numerator_minus   => false, :numerator_min   => 1, :numerator_max   => 1,
                               }.to_json)
  problem_group.save!

  problem_group = ProblemGroup.new(:label => "因数分解",
                                   :description => "与えられた式を因数分解してください。")
  problem_group.problems.build(:unit => Unit.find_by_symbol("factorization"), :level => 0,
                               :amount => 2,
                               :options => {
                                 :order_min => 2, :order_max => 3, :x => %w[x y z],
                                 :factor_minus => true, :factor_min => 0, :factor_max => 10
                               }.to_json)
  problem_group.save!

  problem_group = ProblemGroup.new(:label => "式展開",
                                   :description => "与えられた式を展開してください。")
  problem_group.problems.build(:unit => Unit.find_by_symbol("expansion"), :level => 0,
                               :amount => 2,
                               :options =>  {
                                 :order_min => 2, :order_max => 3, :x => %w[x y z],
                                 :factor_minus => true, :factor_min => 0, :factor_max => 10
                               }.to_json)
  problem_group.save!

  problem_group = ProblemGroup.new(:label => "一次関数",
                                   :description => <<STR)
グラフの P1, P2 はそれぞれ以下のとおりである。
このグラフを表す一次関数の式を y = ax + b の形で書け。
ただし a, b は有理数である。
STR
  problem_group.problems.build(:unit => Unit.find_by_symbol("linear_function_graph"), :level => 0,
                               :amount => 1,
                               :options =>  {
                                 :minus => true,
                                 :x_minus => true, :x_min => 0, :x_max => 10,
                                 :numerator_min => 1, :numerator_max => 10,
                                 :denominator_min => 2, :denominator_max => 5
                               }.to_json)
  problem_group.save!

  problem_group = ProblemGroup.new(:label => "二次方程式",
                                   :description => "与えられた方程式を解きなさい。")
  problem_group.problems.build(:unit => Unit.find_by_symbol("quadratic_equation"), :level => 0,
                               :amount => 2,
                               :options =>  {
                                 :answer_type => :rational,
                                 :r_min => 1,
                                 :r_max => 20,
                                 :p_min => 0,
                                 :p_max => 12,
                                 :q_min => 1,
                                 :q_max => 40,
                               }.to_json)
  problem_group.save!
end

