# -*- coding: utf-8 -*-
module NavigationHelpers
  # Maps a name to a path. Used by the
  #
  #   When /^I go to (.+)$/ do |page_name|
  #
  # step definition in web_steps.rb
  #
  def path_to(page_name)
    case page_name
    
    when /the home\s?page/, /ホーム/
      '/'
    when /the new affiliation page/, /新規所属/
      new_admin_affiliation_path

    when /the new user page/
      new_admin_user_path

    when /the new unit page/, /新規ユニット/
      new_admin_unit_path

    when /the new problem page \((.+?)\)/, /新規問題設定\((.+?)\)/
      new_admin_problem_path :unit_id => Unit.find_by_symbol($1).id

    when /the new problem page/, /新規問題設定/
      new_admin_problem_path

    when /the new problem group page/, /新規問題グループ/
      new_admin_problem_group_path

    # Add more mappings here.
    # Here is an example that pulls values out of the Regexp:
    #
    #   when /^(.*)'s profile page$/i
    #     user_profile_path(User.find_by_login($1))
    when /the login page/, /ログイン/
      url_for(:controller => '/auth')
    when /the admin page/, /管理/
      url_for(:controller => '/admin')
    when /the units index page/, /ユニット一覧/
      admin_units_path
    when /the affiliations index page/, /所属一覧/
      admin_affiliations_path
    when /the users index page/, /ユーザ一覧/
      admin_users_path
    when /the problems index page/, /問題設定一覧/
      admin_problems_path
    when /the problem groups index page/, /問題グループ一覧/
      admin_problem_groups_path
    else
      raise "Can't find mapping from \"#{page_name}\" to a path.\n" +
        "Now, go and add a mapping in #{__FILE__}"
    end
  end
end

World(NavigationHelpers)
