# -*- coding: utf-8 -*-
begin
  require 'vlad'
  Vlad.load :app => :thin, :scm => :git, :web => nil
rescue LoadError
  # do nothing

  # when remote server does not have vlad gem.
  def remote_task(*args)
    # nop
  end
end

namespace :vlad do
  desc "full deploy"
  task :deploy => [
                   :stop_app,
                   :update_src,
                   :update,
                   :update_owner,
                   :update_config,
                   :migrate,
                   :start_app
                  ]

  remote_task :setup_app do
    dirs = [deploy_to, releases_path, scm_path, shared_path]
    dirs += %w(system log pids).map { |d| File.join(shared_path, d) }
    sudo "chown www-data:www-data #{dirs.join(' ')}"
  end

  # Rake.clear_tasks が無いみたいなので直接インスタンス変数を操作した
  Rake.application.instance_variable_get("@tasks").delete("vlad:update")
  Rake.application.instance_variable_get("@tasks").delete("vlad:start_app")
  Rake.application.instance_variable_get("@tasks").delete("vlad:stop_app")
  Rake.application.instance_variable_get("@tasks").delete("vlad:cleanup")

  # sf.jp の Git リポジトリは git archive の --remote で直接指定出来ないため
  task :update_src do
    sh "#{git_cmd} archive --format=tar HEAD | gzip > #{release_name}.tar.gz"
    sh "#{rsync_cmd} #{rsync_flags.join(' ')} #{release_name}.tar.gz #{domain}:#{scm_path}"
  end

  # サーバーから外には git プロトコルが通らないため再定義した
  remote_task :update, :roles => :app do
    symlink = false
    begin
      run [
           "mkdir -p #{release_path}",
           "tar xfz #{scm_path}/#{release_name}.tar.gz -C #{release_path}",
           "chmod -R g+w #{latest_release}",
           "rm -rf #{latest_release}/log #{latest_release}/public/system #{latest_release}/tmp/pids",
           "mkdir -p #{latest_release}/db #{latest_release}/tmp"
          ].join(" && ")
      Rake::Task['vlad:update_symlinks'].invoke
      symlink = true
      run "rm -f #{current_path} && ln -s #{latest_release} #{current_path}"
      run "echo #{now} $USER #{revision} #{File.basename release_path} >> #{deploy_to}/revisions.log"
    rescue => e
      run "rm -f #{current_path} && ln -s #{previous_release} #{current_path}" if symlink
      run "rm -rf #{release_path}"
      raise e
    end
  end

  remote_task :update_owner, :roles => :app do
    sudo "chown -R www-data:www-data #{latest_release}"
  end

  remote_task :update_config, :roles => :app do
    run "cp #{shared_path}/database.yml #{shared_path}/smtp.yml #{release_path}/config/"
  end

  remote_task :start_app, :roles => :app do
    sudo "#{thin_command} -C #{thin_conf} restart"
  end

  remote_task :stop_app, :roles => :app do
    sudo "#{thin_command} -C #{thin_conf} stop"
  end

  # シンボリックリンクの処理をしておかないと削除時にエラーが発生するため追加
  remote_task :cleanup do
    max = keep_releases
    if releases.length <= max then
      puts "no old releases to clean up #{releases.length} <= #{max}"
    else
      puts "keeping #{max} of #{releases.length} deployed releases"

      (releases - releases.last(max)).each do |release|
        path = File.join(releases_path, release)
        run "find #{path} -type l -exec unlink {} \\;"
        run "rm -rf #{path}"
      end
    end
  end
end
