require 'spec_helper'

describe Admin::UsersController do

  before(:each) do
    @user = Factory(:user)
    @admin = Factory(:admin)
    @teacher = Factory(:teacher)
  end

  describe "GET 'index'" do
    describe "by anonymous user" do
      before{ get "index" }
      it{ response.should be_redirect }
    end
    describe "by non-admin user" do
      before do
        login_as(@user)
        get 'index'
      end
      it{ response.status.should == "403 Forbidden" }
      it{ response.should render_template('/admin/error') }
    end
    describe "by teacher" do
      before do
        login_as(@teacher)
        get 'index'
      end
      it{ response.status.should == "403 Forbidden" }
      it{ response.should render_template('/admin/error') }
    end
    describe "by admin" do
      before do
        login_as(@admin)
        get 'index'
      end
      it{ response.should be_success }
      it{ response.should render_template('index') }
    end
  end

  describe "GET 'show'" do
    describe "by anonymous user" do
      before{ get "show", :id => @user }
      it{ response.should be_redirect }
    end
    describe "by non-admin user" do
      before do
        login_as(@user)
        get 'show', :id => @user
      end
      it{ response.status.should == "403 Forbidden" }
      it{ response.should render_template('/admin/error') }
    end
    describe "by teacher" do
      before do
        login_as(@teacher)
        get 'show', :id => @user
      end
      it{ response.status.should == "403 Forbidden" }
      it{ response.should render_template('/admin/error') }
    end
    describe "by admin" do
      before do
        login_as(@admin)
        get 'show', :id => @user
      end
      it{ response.should be_success }
      it{ response.should render_template('show') }
    end
  end

  describe "DELETE 'destroy'" do
    describe "by anonymous user" do
      before{ delete "destroy", :id => @user }
      it{ response.should be_redirect }
    end
    describe "by non-admin user" do
      before do
        login_as(@user)
        delete 'destroy', :id => @user
      end
      it{ response.status.should == "403 Forbidden" }
      it{ response.should render_template('/admin/error') }
    end
    describe "by teacher" do
      before do
        login_as(@teacher)
        delete 'destroy', :id => @user
      end
      it{ response.status.should == "403 Forbidden" }
      it{ response.should render_template('/admin/error') }
    end
    describe "by admin" do
      before do
        login_as(@admin)
        delete 'destroy', :id => @user
      end
      it{ response.should be_redirect }
      it{ response.should redirect_to("/admin/users") }
    end
  end

  describe "DELETE 'destroy_selected'" do
    describe "by anonymous user" do
      before{ delete "destroy_selected" }
      it{ response.should be_redirect }
    end
    describe "by non-admin user" do
      before do
        login_as(@user)
        delete 'destroy_selected'
      end
      it{ response.status.should == "403 Forbidden" }
      it{ response.should render_template('/admin/error') }
    end
    describe "by teacher" do
      before do
        login_as(@teacher)
        delete 'destroy_selected'
      end
      it{ response.status.should == "403 Forbidden" }
      it{ response.should render_template('/admin/error') }
    end
    describe "by admin" do
      before do
        login_as(@admin)
        delete "destroy_selected", :user_ids => ["1", "2", "3"]
      end
      it{ response.should redirect_to(admin_users_url) }
    end
  end
end
