require 'spec_helper'

describe ProblemsController do

  before do
    @user = Factory(:user)
  end

  describe "GET 'quiz'" do
    describe "by anonymous user" do
      before do
        get 'quiz', :id => 1
      end
      it{ response.should be_redirect }
      it{ response.should redirect_to(root_path) }
    end
    describe "by user" do
      before do
        problem = mock(Problem)
        problem.should_receive(:generate).and_return([])
        group = mock(ProblemGroup)
        group.should_receive(:problems).and_return([problem])
        ProblemGroup.should_receive(:find).and_return(group)
        login_as(@user)
        get 'quiz', :id => 1
      end
      it{ response.should be_success }
      it{ response.should render_template('quiz') }
    end
  end

  describe "POST 'answer'" do
    describe "by anonymous user" do
      before do
        post 'answer'
      end
      it{ response.should be_redirect }
      it{ response.should redirect_to(root_path) }
    end
    describe "by user" do
      before do
        login_as(@user)
        session[:prepared_submit_token] = ["xxx"]
        post('answer',
             :problems => [],
             :answers => [],
             :problem_ids => [],
             :problem_group_id => 1,
             :submit_token => "xxx")
      end
      it{ response.should be_redirect }
    end
  end
end
