require 'spec_helper'

describe ReportsController do

  before do
    @user = Factory(:user)
  end

  describe "routes" do
    describe "for index" do
      before do
        @base = { :controller => 'reports', :action => 'index' }
      end
      [
       [{ :year => '2010', :month => '02', :day => '01' }, '/reports/2010/02/01'],
       [{ :year => '2010', :month => '2', :day => '1' }, '/reports/2010/2/1'],
      ].each do |options, expected|
        it{ route_for(@base.merge(options)).should == expected }
      end
    end
    describe "for range" do
      before do
        @base = { :controller => 'reports', :action => 'range' }
      end
      [
       [{ :range => '20100201-20100228' }, '/reports/20100201-20100228'],
      ].each do |options, expected|
        it{ route_for(@base.merge(options)).should == expected }
      end
    end
    describe "for show" do
      before do
        @base = { :controller => 'reports', :action => 'show' }
      end
      [
       [{ :id => '1' }, '/reports/1'],
       [{ :id => '123' }, '/reports/123'],
       [{ :id => '20091111' }, '/reports/20091111'],
      ].each do |options, expected|
        it{ route_for(@base.merge(options)).should == expected }
      end
    end
    describe "params from" do
      before do
        @base = { :controller => "reports" }
      end
      [
       ["/reports/1", { :action => "show", :id => "1" }],
       ["/reports/2009/10/10", { :action => "index", :year => "2009", :month => "10", :day => "10" }],
       ["/reports/20091010-20091111", { :action => "range", :range => "20091010-20091111" }],
       ["/reports/range?start=2010-02-01&end=2010-02-02&commit=Search",
        { :action => 'range', :start => "2010-02-01", :end => "2010-02-02", :commit => "Search" }]
      ].each do |path, options|
          it{ params_from(:get, path).should ==  @base.merge(options) }
      end
    end
  end

  describe "GET 'index'" do
    describe "by anonymous user" do
      before do
        get 'index', :year => 2010, :month => 2, :day => 22
      end
      it{ response.should be_redirect }
      it{ response.should redirect_to(root_path) }
    end
    describe "by user" do
      before do
        login_as(@user)
        get 'index', :year => 2010, :month => 2, :day => 22
      end
      it{ response.should be_success }
      it{ response.should render_template("index") }
    end
  end

  describe "GET range" do
    describe "by anonymous user" do
      before do
        get 'range'
      end
      it{ response.should be_redirect }
      it{ response.should redirect_to(root_path) }
    end
    describe "by user" do
      describe "when set range" do
        before do
          login_as(@user)
          get 'range', :range => '20100201-20100228'
        end
        it{ response.should be_success }
        it{ response.should render_template("range") }
      end
    end
  end
end

