require 'spec_helper'

describe UsersController do
  before(:each) do
    @user = Factory(:user)
    attrs = {
      :name           => 'name',
      :email          => 'name@example.com',
      :admission_date => Date.today,
      :lock_version   => 0,
    }
    @edit_form = UserEditForm.new(attrs)
  end

  describe "GET 'show'" do
    describe "by anonymous user" do
      before{ get 'show' }
      it{ response.should be_redirect }
    end
    describe "by login user" do
      before do
        login_as(@user)
        get 'show'
      end
      it{ response.should be_success }
      it{ assigns[:user].should == @user }
      it{ response.should render_template('show') }
    end
  end

  describe "GET 'edit'" do
    describe "by anonymous user" do
      before do
        get 'edit'
      end
      it{ response.should be_redirect }
    end
    describe "by login user" do
      before do
        login_as(@user)
        get 'edit'
      end
      it{ response.should be_success }
    end
  end

  describe "PUT 'update'" do
    describe 'by login user' do
      before do
        login_as(@user)
        put 'update', :user => @edit_form.attributes
      end
      it{ response.should be_redirect }
    end
  end

end
