# -*- coding: utf-8 -*-
require 'spec_helper'

class FooEditForm < ActiveForm
  acts_as_problem
end

class BarEditForm < ActiveForm
  acts_as_problem :operators => true, :term_number => true
end

class BazEditForm < ActiveForm
  acts_as_problem :operators => %w[plus minus fraction], :term_number => true
end

describe ActsAsProblem do
  describe "without options" do
    before do
      @valid_attributes = {
        :unit_id      => 1,
        :level        => 1,
        :amount       => 2,
        :lock_version => 0,
      }
      @klass = FooEditForm
      @basic = @klass.new(@valid_attributes)
    end

    context "klass" do
      it{ @klass.column_names.should == %w[unit_id level amount lock_version]}
    end

    context "basic" do
      it{ @basic.should be_valid }
    end

    context "validation" do

      context "unit_id" do
        context "is nil" do
          subject{ @basic.unit_id = nil; @basic }
          it{ should_not be_valid }
        end
        context "is not nnumber" do
          subject{ @basic.unit_id = "a"; @basic }
          it{ should_not be_valid}
        end
      end

      context "level" do
        context "is nil" do
          subject{ @basic.level = nil; @basic }
          it{ should_not be_valid }
        end
        context "is -1" do
          subject{ @basic.level = -1; @basic }
          it{ should_not be_valid }
        end
        context "is 0" do
          subject{ @basic.level = 0; @basic }
          it{ should be_valid }
        end
        context "is 100" do
          subject{ @basic.level = 100; @basic }
          it{ should be_valid }
        end
        context "is 101" do
          subject{ @basic.level = 101; @basic }
          it{ should_not be_valid }
        end
      end

      context "amount" do
        context "is nil" do
          subject{ @basic.amount = nil; @basic }
          it{ should_not be_valid }
        end
        context "is 0" do
          subject{ @basic.amount = 0; @basic }
          it{ should_not be_valid }
        end
        context "is 1" do
          subject{ @basic.amount = 1; @basic }
          it{ should be_valid }
        end
        context "is 100" do
          subject{ @basic.amount = 100; @basic }
          it{ should be_valid }
        end
        context "is 101" do
          subject{ @basic.amount = 101; @basic }
          it{ should_not be_valid }
        end
      end
    end

    context "operators" do
      it{ @basic.operators.should == [] }
    end
  end

  describe "with options" do
    before do
      @valid_attributes = {
        :unit_id         => 1,
        :level           => 1,
        :amount          => 2,
        :lock_version    => 0,
        :term_number     => 2,
        :operators_plus  => true,
        :operators_minus => true,
        :operators_times => true,
        :operators_div   => true,
      }
      @klass = BarEditForm
      @basic = @klass.new(@valid_attributes)
    end

    context "klass" do
      it{
        @klass.column_names.should ==
        %w[unit_id level amount lock_version
           operators_plus operators_minus operators_times operators_div term_number]
      }
    end

    context "basic" do
      it{ @basic.should be_valid }
    end

    context "validation" do
      context "term_number" do
        context "is string" do
          subject{ @basic.term_number = "a"; @basic }
          it{ should_not be_valid }
        end
        context "is nil" do
          subject{ @basic.term_number = nil; @basic }
          it{ should_not be_valid }
        end
        context "is empty" do
          subject{ @basic.term_number = ""; @basic }
          it{ should_not be_valid }
        end
        context "is float" do
          subject{ @basic.term_number = 1.5; @basic }
          it{ should_not be_valid }
        end
        context "is 1" do
          subject{ @basic.term_number = 1; @basic }
          it{ should_not be_valid }
        end
        context "is 2" do
          subject{ @basic.term_number = 2; @basic }
          it{ should be_valid }
        end
      end

      context "operators" do
        context "no operators specified" do
          # 特殊な validation はクラスごとに指定する
          subject{
            @basic.operators_plus = false
            @basic.operators_minus = false
            @basic.operators_times = false
            @basic.operators_div = false
            @basic
          }
          it{ should be_valid }
        end
      end
    end

    context "operators" do
      it{ @basic.operators.should == %w[+ - * div] }
    end

    context "operators=" do
      subject{ @basic.operators = %w[+ -]; @basic }
      its(:operators_plus){ should be_true }
      its(:operators_minus){ should be_true }
      its(:operators_times){ should_not be_true }
      its(:operators_div){ should_not be_true }
    end
  end

  describe "with options (operators is Array)" do
    before do
      @valid_attributes = {
        :unit_id            => 1,
        :level              => 1,
        :amount             => 2,
        :lock_version       => 0,
        :term_number        => 2,
        :operators_plus     => true,
        :operators_minus    => true,
        :operators_fraction => true,
      }
      @klass = BazEditForm
      @basic = @klass.new(@valid_attributes)
    end

    context "klass" do
      it{
        @klass.column_names.should ==
        %w[unit_id level amount lock_version
           operators_plus operators_minus operators_fraction term_number]
      }
    end

    context "basic" do
      it{ @basic.should be_valid }
    end

    context "validation" do
      context "operators" do
        context "no operators specified" do
          # 特殊な validation はクラスごとに指定する
          subject{
            @basic.operators_plus = false
            @basic.operators_minus = false
            @basic.operators_fraction = false
            @basic
          }
          it{ should be_valid }
        end
      end
    end

    context "operators" do
      it{ @basic.operators.should == %w[+ - /] }
      context "set false" do
        subject{ @basic.operators_plus = false ; @basic.operators }
        it{ should == %w[- /] }
      end
    end

    context "operators=" do
      subject{ @basic.operators = %w[+ -]; @basic }
      its(:operators_plus){ should be_true }
      its(:operators_minus){ should be_true }
      its(:operators_fraction){ should_not be_true }
    end
  end
end
