require 'spec_helper'

describe ComplexFractionalArithmeticEditForm do
  before do
    @valid_attributes = {
      :unit_id              => 1,
      :level                => 1,
      :amount               => 2,
      :numerator_term_min   => 1,
      :numerator_term_max   => 2,
      :denominator_term_min => 1,
      :denominator_term_max => 2,
      :operators_plus       => true,
      :operators_minus      => true,
      :operators_times      => true,
    }
    @klass = ComplexFractionalArithmeticEditForm
    @basic = @klass.new(@valid_attributes)
  end

  context "basic" do
    it{ @basic.should be_valid }
  end

  context "validation" do

    [
     [:numerator_term_min, 0, 1],
     [:numerator_term_max, 0, 1],
     [:denominator_term_min, 0, 1],
     [:denominator_term_max, 0, 1],
    ].each do |target, ng, ok|
      context target do
        before do
          @target = target
          @boundary_value_ng = ng
          @boundary_value_ok = ok
        end
        it_should_behave_like 'validates_presence_of'
        it_should_behave_like 'validates_numericality_of'
      end
    end

    context "relation" do
      context "numerator_term" do
        before do
          @target_left = :numerator_term_min
          @target_right = :numerator_term_max
          @message = "Numerator term min must be greater than Numerator term max."
        end
        it_should_behave_like "check_min_max"
      end
      context "denominator_term" do
        before do
          @target_left = :denominator_term_min
          @target_right = :denominator_term_max
          @message = "Denominator term min must be greater than Denominator term max."
        end
        it_should_behave_like "check_min_max"
      end
    end

    context "operators" do
      context "no operators specified" do
        subject{
          @basic.operators_plus = false
          @basic.operators_minus = false
          @basic.operators_times = false
          @basic
        }
        it{ should_not be_valid }
      end
    end
  end

  context "options_hash" do
    subject{ @basic.options_hash }
    it{
      should == {
        :numerator_term_min => 1, :numerator_term_max => 2,
        :denominator_term_min => 1, :denominator_term_max => 2,
        :operators => %w[+ - *]
      }
    }
  end

  context "attributes_for_model" do
    subject{ @basic.attributes_for_model }
    it{
      should == {
        :unit_id => 1,
        :level   => 1,
        :amount  => 2,
        :options => @basic.options_hash.to_json,
        :lock_version => 0
      }
    }
  end

  context "set_options" do
    context "basic" do
      subject{
        options = {
          :numerator_term_min => 2, :numerator_term_max => 3,
          :denominator_term_min => 3, :denominator_term_max => 3,
          :operators => %w[+ - *]
        }
        @basic.set_options(options.to_json)
        @basic
      }
      its(:numerator_term_min){ should == 2 }
      its(:numerator_term_max){ should == 3 }
      its(:denominator_term_min){ should == 3 }
      its(:denominator_term_max){ should == 3 }
      its(:operators_plus){ should be_true }
      its(:operators_minus){ should be_true }
      its(:operators_times){ should be_true }
    end
    context "set nothing" do
      it "should not raise Exception" do
        lambda{ @basic.set_options({}.to_json) }.should_not raise_exception
      end
    end
  end
end
