require 'spec_helper'

describe ComplexNumberArithmeticEditForm do
  before do
    @valid_attributes = {
      :unit_id            => 1,
      :level              => 1,
      :amount             => 2,
      :min                => 0,
      :max                => 100,
      :fractional_mode    => false,
      :term_number        => 2,
      :operators_plus     => true,
      :operators_minus    => true,
      :operators_times    => true,
      :operators_fraction => true,
    }
    @klass = ComplexNumberArithmeticEditForm
    @basic = @klass.new(@valid_attributes)
  end

  context "basic" do
    it{ @basic.should be_valid }
  end

  context "validation" do

    [
     [:min, -1, 0],
     [:max, -1, 0],
    ].each do |target, ng, ok|
      context target do
        before do
          @target = target
          @boundary_value_ng = ng
          @boundary_value_ok = ok
        end
        it_should_behave_like 'validates_presence_of'
        it_should_behave_like 'validates_numericality_of'
      end
    end

    context "relation" do
      before do
        @target_left = :min
        @target_right = :max
        @message = "Min must be greater than Max."
      end
      it_should_behave_like "check_min_max"
    end

    context "operators" do
      context "no operators specified" do
        subject{
          @basic.operators_plus = false
          @basic.operators_minus = false
          @basic.operators_times = false
          @basic.operators_fraction = false
          @basic
        }
        it{ should_not be_valid }
      end
    end
  end

  context "options_hash" do
    subject{ @basic.options_hash }
    it{
      should == {
        :min => 0, :max => 100,
        :fractional_mode => false,
        :term_number => 2,
        :operators => %w[+ - * /]
      }
    }
  end

  context "attributes_for_model" do
    subject{ @basic.attributes_for_model }
    it{
      should == {
        :unit_id => 1,
        :level   => 1,
        :amount  => 2,
        :options => @basic.options_hash.to_json,
        :lock_version => 0
      }
    }
  end

  context "set_options" do
    context "basic" do
      subject{
        options = {
          :min => 1, :max => 99,
          :fractional_mode => true,
          :term_number => 3,
          :operators => %w[+ - * /]
        }
        @basic.set_options(options.to_json)
        @basic
      }
      its(:min){ should == 1 }
      its(:max){ should == 99 }
      its(:fractional_mode){ should be_true }
      its(:term_number){ should == 3 }
      its(:operators_plus){ should be_true }
      its(:operators_minus){ should be_true }
      its(:operators_times){ should be_true }
      its(:operators_fraction){ should be_true }
    end
    context "set nothing" do
      it "should not raise Exception" do
        lambda{ @basic.set_options({}.to_json) }.should_not raise_exception
      end
    end
  end
end
