require 'spec_helper'

describe FactorizationEditForm do
  before do
    @valid_attributes = {
      :unit_id      => 1,
      :level        => 1,
      :amount       => 2,
      :order_min    => 2,
      :order_max    => 3,
      :x            => "x",
      :factor_minus => true,
      :factor_min   => 0,
      :factor_max   => 10,
    }
    @klass = FactorizationEditForm
    @basic = @klass.new(@valid_attributes)
  end

  context "basic" do
    it{ @basic.should be_valid }
  end

  context "validation" do

    [
     [:order_min, 1, 2],
     [:order_max, 1, 2],
     [:factor_min, -1, 0],
     [:factor_max, -1, 0],
    ].each do |target, ng, ok|
      context target do
        before do
          @target = target
          @boundary_value_ng = ng
          @boundary_value_ok = ok
        end
        it_should_behave_like 'validates_presence_of'
        it_should_behave_like 'validates_numericality_of'
      end
    end

    context "x" do
      before do
        @target = :x
      end
      it_should_behave_like 'validates_presence_of'
      context "is 'a,b,c'" do
        subject{ @basic.x = 'a,b,c'; @basic }
        it{ should be_valid }
      end
      context "is 'a, b, c'" do
        subject{ @basic.x = 'a, b, c'; @basic }
        it{ should be_valid }
      end
      context "is '1, 2, 3'" do
        subject{ @basic.x = '1, 2, 3'; @basic }
        it{ should_not be_valid }
      end
      context "is 'aa, bb, cc'" do
        subject{ @basic.x = 'aa, bb, cc'; @basic }
        it{ should_not be_valid }
      end
    end

    context "relation" do
      context "order" do
        before do
          @target_left = :order_min
          @target_right = :order_max
          @message = "Order min must be greater than Order max."
        end
        it_should_behave_like "check_min_max"
      end
      context "factor" do
        before do
          @target_left = :factor_min
          @target_right = :factor_max
          @message = "Factor min must be greater than Factor max."
        end
        it_should_behave_like "check_min_max"
      end
    end
  end

  context "options_hash" do
    subject{ @basic.options_hash }
    it{
      should == {
        :order_min => 2, :order_max => 3, :x => ["x"],
        :factor_minus => true, :factor_min => 0, :factor_max => 10
      }
    }
  end

  context "set_options" do
    context "basic" do
      subject{
        options = {
          :order_min => 2, :order_max => 3, :x => %w[x y z],
          :factor_minus => true, :factor_min => 10, :factor_max => 100
        }
        @basic.set_options(options.to_json)
        @basic
      }
      its(:order_min){ should == 2 }
      its(:order_max){ should == 3 }
      its(:x){ should == %w[x y z]  }
      its(:factor_minus){ should be_true }
      its(:factor_min){ should == 10 }
      its(:factor_max){ should == 100 }
    end
    context "set nothing" do
      it "should not raise Exception" do
        lambda{ @basic.set_options({}.to_json) }.should_not raise_exception
      end
    end
  end
end
