require 'spec_helper'

describe FractionalArithmeticEditForm do
  before do
    @valid_attributes = {
      :unit_id         => 1,
      :level           => 1,
      :amount          => 2,
      :minus           => true,
      :numerator_min   => 1,
      :numerator_max   => 100,
      :denominator_min => 2,
      :denominator_max => 100,
      :term_number     => 2,
      :operators_plus  => true,
      :operators_minus => true,
      :operators_times => true,
      :operators_div   => true,
    }
    @klass = FractionalArithmeticEditForm
    @basic = @klass.new(@valid_attributes)
  end

  context "basic" do
    it{ @basic.should be_valid }
  end

  context "validation" do
    [
     [:numerator_min, 0, 1],
     [:numerator_max, 0, 1],
     [:denominator_min, 1, 2],
     [:denominator_max, 1, 2],
    ].each do |target, ng, ok|
      context target do
        before do
          @target = target
          @boundary_value_ng = ng
          @boundary_value_ok = ok
        end
        it_should_behave_like 'validates_presence_of'
        it_should_behave_like 'validates_numericality_of'
      end
    end

    context "relation" do
      context "numerator" do
        before do
          @target_left = :numerator_min
          @target_right = :numerator_max
          @message = "Numerator min must be greater than Numerator max."
        end
        it_should_behave_like "check_min_max"
      end

      context "denominator" do
        before do
          @target_left = :denominator_min
          @target_right = :denominator_max
          @message = "Denominator min must be greater than Denominator max."
        end
        it_should_behave_like "check_min_max"
      end
    end

    context "operators" do
      context "no operators specified" do
        subject{
          @basic.operators_plus = false
          @basic.operators_minus = false
          @basic.operators_times = false
          @basic.operators_div = false
          @basic
        }
        it{ should_not be_valid }
      end
    end
  end

  context "options_hash" do
    subject{ @basic.options_hash }
    it{
      should == {
        :minus => true,
        :numerator_min   => 1,
        :numerator_max   => 100,
        :denominator_min => 2,
        :denominator_max => 100,
        :term_number => 2,
        :operators => %w[+ - * div]
      }
    }
  end

  context "set_options" do
    context "basic" do
      subject{
        options = {
          :minus => true,
          :numerator_min => 2, :numerator_max => 10,
          :denominator_min => 4, :denominator_max => 11,
          :term_number => 3,
          :operators => %w[+ - * div]
        }
        @basic.set_options(options.to_json)
        @basic
      }
      its(:minus){ should be_true }
      its(:numerator_min){ should == 2 }
      its(:numerator_max){ should == 10 }
      its(:denominator_min){ should == 4 }
      its(:denominator_max){ should == 11 }
      its(:operators_plus){ should be_true }
      its(:operators_minus){ should be_true }
      its(:operators_times){ should be_true }
      its(:operators_div){ should be_true }
    end
    context "set nothing" do
      it "should not raise Exception" do
        lambda{ @basic.set_options({}.to_json) }.should_not raise_exception
      end
    end
  end
end
