require 'spec_helper'

describe FractionalExpressionArithmeticEditForm do
  before do
    @valid_attributes = {
      :unit_id              => 1,
      :level                => 1,
      :amount               => 2,
      :x                    => "x",
      :numerator_term_min   => 1,
      :numerator_term_max   => 2,
      :denominator_term_min => 1,
      :denominator_term_max => 2,
      :factor_minus         => true,
      :factor_min           => 0,
      :factor_max           => 100,
    }
    @klass = FractionalExpressionArithmeticEditForm
    @basic = @klass.new(@valid_attributes)
  end

  context "basic" do
    it{ @basic.should be_valid }
  end

  context "validation" do

    [
     [:numerator_term_min, 0, 1],
     [:numerator_term_max, 0, 1],
     [:denominator_term_min, 0, 1],
     [:denominator_term_max, 0, 1],
     [:factor_min, -1, 0],
     [:factor_max, -1, 0],
    ].each do |target, ng, ok|
      context target do
        before do
          @target = target
          @boundary_value_ng = ng
          @boundary_value_ok = ok
        end
        it_should_behave_like 'validates_presence_of'
        it_should_behave_like 'validates_numericality_of'
      end
    end

    context "relation" do
      context "numerator_term" do
        before do
          @target_left = :numerator_term_min
          @target_right = :numerator_term_max
          @message = "Numerator term min must be greater than Numerator term max."
        end
        it_should_behave_like "check_min_max"
      end

      context "denominator_term_number" do
        before do
          @target_left = :denominator_term_min
          @target_right = :denominator_term_max
          @message = "Denominator term min must be greater than Denominator term max."
        end
        it_should_behave_like "check_min_max"
      end

      context "factor" do
        before do
          @target_left = :factor_min
          @target_right = :factor_max
          @message = "Factor min must be greater than Factor max."
        end
        it_should_behave_like "check_min_max"
      end
    end
  end

  context "options_hash" do
    subject{ @basic.options_hash }
    it{
      should == {
        :x                    => ["x"],
        :numerator_term_min   => 1,
        :numerator_term_max   => 2,
        :denominator_term_min => 1,
        :denominator_term_max => 2,
        :factor_minus         => true,
        :factor_min           => 0,
        :factor_max           => 100,
      }
    }
  end

  context "set_options" do
    context "basic" do
      subject{
        options = {
          :x                    => ["y"],
          :numerator_term_min   => 2,
          :numerator_term_max   => 3,
          :denominator_term_min => 2,
          :denominator_term_max => 3,
          :factor_minus         => true,
          :factor_min           => 1,
          :factor_max           => 99,
        }
        @basic.set_options(options.to_json)
        @basic
      }
      its(:x){ should == ["y"] }
      its(:numerator_term_min){ should == 2 }
      its(:numerator_term_max){ should == 3 }
      its(:denominator_term_min){ should == 2 }
      its(:denominator_term_max){ should == 3 }
      its(:factor_minus){ should be_true }
      its(:factor_min){ should == 1 }
      its(:factor_max){ should == 99 }
    end
    context "set nothing" do
      it "should not raise Exception" do
        lambda{ @basic.set_options({}.to_json) }.should_not raise_exception
      end
    end
  end
end
