# -*- coding: utf-8 -*-
require 'spec_helper'

describe LearningResult do
  before(:each) do
    @valid_attributes = {
      :problem_id         => 1,
      :report_id          => 1,
      :problem_expression => "",
      :answer             => "",
      :example_solution   => "",
      :correct            => true
    }
    @klass = LearningResult
    @basic = @klass.new(@valid_attributes)
  end

  describe "basic" do
    it{ @basic.should be_valid }
  end

  describe "validattion" do
    describe "problem_id" do
      describe "is nil" do
        subject{ @basic.problem_id = nil; @basic }
        it{ should_not be_valid }
      end
    end
    describe "report_id" do
      describe "is nil" do
        subject{ @basic.report_id = nil; @basic }
        it{ should_not be_valid }
      end
    end
  end

  describe "callbacks" do
    describe "before_save" do
      before do
        user = Factory(:user)
        problem = Factory(:ordinary_arithmetic_level_1)
        solver = mock('solver')
        answer = mock('answer')
        answer.should_receive(:normalize).at_least(1).and_return("2")
        problem.should_receive(:solver).and_return(solver)
        solver.should_receive(:solve).and_return(answer)
        @learning_result = LearningResult.create!(:problem => problem,
                                                  :report_id => 1,
                                                  :problem_expression => "1+1",
                                                  :answer => "2")
      end
      it "set example_solution" do
        @learning_result.should_not be_blank
      end
      it "set correct" do
        @learning_result.should be_true
      end
    end
  end

end
