require 'spec_helper'

describe LinearFunctionGraphEditForm do
  before do
    @valid_attributes = {
      :unit_id         => 1,
      :level           => 1,
      :amount          => 2,
      :minus           => true,
      :x_minus         => true,
      :x_min           => 0,
      :x_max           => 10,
      :numerator_min   => 1,
      :numerator_max   => 100,
      :denominator_min => 1,
      :denominator_max => 100,
    }
    @klass = LinearFunctionGraphEditForm
    @basic = @klass.new(@valid_attributes)
  end

  context "basic" do
    it{ @basic.should be_valid }
  end

  context "validation" do

    [
     [:x_min, -1, 0],
     [:x_max, -1, 0],
     [:numerator_min, 0, 1],
     [:numerator_max, 0, 1],
     [:denominator_min, 0, 1],
     [:denominator_max, 0, 1],
    ].each do |target, ng, ok|
      context target do
        before do
          @target = target
          @boundary_value_ng = ng
          @boundary_value_ok = ok
        end
        it_should_behave_like 'validates_presence_of'
        it_should_behave_like 'validates_numericality_of'
      end
    end

    context "relation" do
      context "x" do
        before do
          @target_left = :x_min
          @target_right = :x_max
          @message = "X min must be greater than X max."
        end
        it_should_behave_like "check_min_max"
      end

      context "numerator" do
        before do
          @target_left = :numerator_min
          @target_right = :numerator_max
          @message = "Numerator min must be greater than Numerator max."
        end
        it_should_behave_like "check_min_max"
      end

      context "denominator" do
        before do
          @target_left = :denominator_min
          @target_right = :denominator_max
          @message = "Denominator min must be greater than Denominator max."
        end
        it_should_behave_like "check_min_max"
      end
    end
  end

  context "options_hash" do
    subject{ @basic.options_hash }
    it{
      should == {
        :minus           => true,
        :x_minus         => true,
        :x_min           => 0,
        :x_max           => 10,
        :numerator_min   => 1,
        :numerator_max   => 100,
        :denominator_min => 1,
        :denominator_max => 100,
      }
    }
  end

  context "set_options" do
    context "basic" do
      subject{
        options = {
          :minus => true,
          :x_minus => true, :x_min => 0, :x_max => 10,
          :numerator_min => 2, :numerator_max => 10,
          :denominator_min => 4, :denominator_max => 11,
        }
        @basic.set_options(options.to_json)
        @basic
      }
      its(:minus){ should be_true }
      its(:x_minus){ should be_true }
      its(:x_min){ should == 0 }
      its(:x_max){ should == 10 }
      its(:numerator_min){ should == 2 }
      its(:numerator_max){ should == 10 }
      its(:denominator_min){ should == 4 }
      its(:denominator_max){ should == 11 }
    end
    context "set nothing" do
      it "should not raise Exception" do
        lambda{ @basic.set_options({}.to_json) }.should_not raise_exception
      end
    end
  end
end
