require 'spec_helper'

describe PartialFractionExpansionEditForm do
  before do
    @valid_attributes = {
      :unit_id           => 1,
      :level             => 1,
      :amount            => 2,
      :order_min         => 2,
      :order_max         => 3,
      :x                 => "x",
      :factor_minus      => true,
      :factor_min        => 0,
      :factor_max        => 10,
      :coefficient_minus => true,
      :coefficient_min   => 1,
      :coefficient_max   => 10,
      :numerator_minus   => true,
      :numerator_min     => 1,
      :numerator_max     => 10,
    }
    @klass = PartialFractionExpansionEditForm
    @basic = @klass.new(@valid_attributes)
  end

  context "basic" do
    it{ @basic.should be_valid }
  end

  context "validation" do

    [
     [:order_min, 1, 2],
     [:order_max, 1, 2],
     [:factor_min, -1, 0],
     [:factor_max, -1, 0],
     [:coefficient_min, 0, 1],
     [:coefficient_max, 0, 1],
     [:numerator_min, 0, 1],
     [:numerator_max, 0, 1],
    ].each do |target, ng, ok|
      context target do
        before do
          @target = target
          @boundary_value_ng = ng
          @boundary_value_ok = ok
        end
        it_should_behave_like 'validates_presence_of'
        it_should_behave_like 'validates_numericality_of'
      end
    end

    context "x" do
      before do
        @target = :x
      end
      it_should_behave_like 'validates_presence_of'
      context "is 'a,b,c'" do
        subject{ @basic.x = 'a,b,c'; @basic }
        it{ should be_valid }
      end
      context "is 'a, b, c'" do
        subject{ @basic.x = 'a, b, c'; @basic }
        it{ should be_valid }
      end
      context "is '1, 2, 3'" do
        subject{ @basic.x = '1, 2, 3'; @basic }
        it{ should_not be_valid }
      end
      context "is 'aa, bb, cc'" do
        subject{ @basic.x = 'aa, bb, cc'; @basic }
        it{ should_not be_valid }
      end
    end

    context "relation" do
      context "order" do
        before do
          @target_left = :order_min
          @target_right = :order_max
          @message = "Order min must be greater than Order max."
        end
        it_should_behave_like "check_min_max"
      end

      context "factor" do
        before do
          @target_left = :factor_min
          @target_right = :factor_max
          @message = "Factor min must be greater than Factor max."
        end
        it_should_behave_like "check_min_max"
      end

      context "coefficient" do
        before do
          @target_left = :coefficient_min
          @target_right = :coefficient_max
          @message = "Coefficient min must be greater than Coefficient max."
        end
        it_should_behave_like "check_min_max"
      end

      context "numerator" do
        before do
          @target_left = :numerator_min
          @target_right = :numerator_max
          @message = "Numerator min must be greater than Numerator max."
        end
        it_should_behave_like "check_min_max"
      end
    end
  end

  context "options_hash" do
    subject{ @basic.options_hash }
    it{
      should == {
        :order_min => 2, :order_max => 3, :x => ["x"],
        :factor_minus => true, :factor_min => 0, :factor_max => 10,
        :coefficient_minus => true, :coefficient_min => 1, :coefficient_max => 10,
        :numerator_minus => true, :numerator_min => 1, :numerator_max => 10,
      }
    }
  end

  context "set_options" do
    context "basic" do
      subject{
        options = {
          :order_min => 2, :order_max => 3, :x => %w[a b c],
          :factor_minus => true, :factor_min => 11, :factor_max => 101,
          :coefficient_minus => true, :coefficient_min => 11, :coefficient_max => 101,
          :numerator_minus => true, :numerator_min => 11, :numerator_max => 101,
        }
        @basic.set_options(options.to_json)
        @basic
      }
      its(:order_min){ should == 2 }
      its(:order_max){ should == 3 }
      its(:x){ should == %w[a b c] }
      its(:factor_minus){ should be_true }
      its(:factor_min){ should == 11 }
      its(:factor_max){ should == 101 }
      its(:coefficient_minus){ should be_true }
      its(:coefficient_min){ should == 11 }
      its(:coefficient_max){ should == 101 }
      its(:numerator_minus){ should be_true }
      its(:numerator_min){ should == 11 }
      its(:numerator_max){ should == 101 }
    end
    context "set nothing" do
      it "should not raise Exception" do
        lambda{ @basic.set_options({}.to_json) }.should_not raise_exception
      end
    end
  end
end
