require 'spec_helper'

describe Problem do
  before(:each) do
    @valid_attributes = {
      :unit_id => 1,
      :level   => 1,
      :amount  => 2,
      :options => "options",
    }
    @klass = Problem
    @basic = @klass.new(@valid_attributes)
  end

  describe "basic" do
    it{ @basic.should be_valid }
  end

  describe "validation" do

    describe "unit_id" do
      describe "is nil" do
        subject{ @basic.unit_id = nil; @basic }
        it{ should_not be_valid }
      end
    end

    describe "level" do
      describe "is nil" do
        subject{ @basic.level = nil; @basic }
        it{ should_not be_valid }
      end
      describe "is -1" do
        subject{ @basic.level = -1; @basic }
        it{ should_not be_valid }
      end
      describe "is 0" do
        subject{ @basic.level = 0; @basic }
        it{ should be_valid }
      end
      describe "is 100" do
        subject{ @basic.level = 100; @basic }
        it{ should be_valid }
      end
      describe "is 101" do
        subject{ @basic.level = 101; @basic }
        it{ should_not be_valid }
      end
    end

    describe "amount" do
      describe "is nil" do
        subject{ @basic.amount = nil; @basic }
        it{ should_not be_valid }
      end
      describe "is -1" do
        subject{ @basic.amount = 0; @basic }
        it{ should_not be_valid }
      end
      describe "is 0" do
        subject{ @basic.amount = 1; @basic }
        it{ should be_valid }
      end
      describe "is 100" do
        subject{ @basic.amount = 100; @basic }
        it{ should be_valid }
      end
      describe "is 101" do
        subject{ @basic.amount = 101; @basic }
        it{ should_not be_valid }
      end
    end

    describe "options" do
      describe "is nil" do
        subject{ @basic.options = nil; @basic }
        it{ should_not be_valid }
      end
      describe "is empty" do
        subject{ @basic.options = ""; @basic }
        it{ should_not be_valid }
      end
    end
  end

  describe "generate" do
    describe "expression" do
      before do
        generator = mock('generator')
        generator.should_receive(:generate).and_return(["1+1"])
        Mint::Generator::Factory.should_receive(:create).and_return(generator)
        @expression = mock('expression')
        Mint::Builder.should_receive(:build).at_least(1).and_return(@expression)
        @basic.options = { }.to_json
        @problem = @basic.dup
        @problem.unit = Factory(:ordinary_arithmetic)
      end
      it{ @problem.generate.first.should == @expression }
    end
    describe "graph" do
      before do
        generator = mock('generator')
        generator.should_receive(:generate).and_return([{ }])
        Mint::Generator::Factory.should_receive(:create).and_return(generator)
        @basic.options = { }.to_json
        @problem = @basic.dup
        @problem.unit = Factory(:linear_function_graph)
      end
      it{ @problem.generate.first.should == { } }
    end
  end
end
