require 'spec_helper'

describe QuadraticEquationEditForm do
  before do
    @valid_attributes = {
      :unit_id     => 1,
      :level       => 1,
      :amount      => 2,
      :answer_type => "rational",
      :r_min       => 1,
      :r_max       => 10,
      :p_minus     => true,
      :p_min       => 0,
      :p_max       => 20,
      :q_min       => 0,
      :q_max       => 30,
    }
    @klass = QuadraticEquationEditForm
    @basic = @klass.new(@valid_attributes)
  end

  context "basic" do
    it{ @basic.should be_valid }
  end

  context "validation" do

    context "answer_type" do
      context "is nil" do
        subject{ @basic.answer_type = nil; @basic }
        it{ should_not be_valid }
      end
      context "is empty" do
        subject{ @basic.answer_type = ''; @basic }
        it{ should_not be_valid }
      end
      context "is xxx" do
        subject{ @basic.answer_type = "xxx"; @basic }
        it{ should_not be_valid }
      end
    end

    [
     [:p_min, -1, 0],
     [:p_max, -1, 0],
     [:q_min, -1, 0],
     [:q_max, -1, 0],
     [:r_min, 0, 1],
     [:r_max, 0, 1],
    ].each do |target, ng, ok|
      context target do
        before do
          @target = target
          @boundary_value_ng = ng
          @boundary_value_ok = ok
        end
        it_should_behave_like 'validates_presence_of'
        it_should_behave_like 'validates_numericality_of'
      end
    end

    context "relation" do
      context "p" do
        before do
          @target_left = :p_min
          @target_right = :p_max
          @message = "P min must be greater than P max."
        end
        it_should_behave_like "check_min_max"
      end

      context "q" do
        before do
          @target_left = :q_min
          @target_right = :q_max
          @message = "Q min must be greater than Q max."
        end
        it_should_behave_like "check_min_max"
      end

      context "r" do
        before do
          @target_left = :r_min
          @target_right = :r_max
          @message = "R min must be greater than R max."
        end
        it_should_behave_like "check_min_max"
      end
    end
  end

  context "options_hash" do
    subject{ @basic.options_hash }
    it{
      should == {
        :answer_type => "rational",
        :r_min       => 1,
        :r_max       => 10,
        :p_minus     => true,
        :p_min       => 0,
        :p_max       => 20,
        :q_min       => 0,
        :q_max       => 30,
      }
    }
  end

  context "set_options" do
    context "basic" do
      subject{
        options = {
          :answer_type => "irrational",
          :r_min       => 1,
          :r_max       => 11,
          :p_minus     => true,
          :p_min       => 2,
          :p_max       => 21,
          :q_min       => 2,
          :q_max       => 31,
        }
        @basic.set_options(options.to_json)
        @basic
      }
      its(:answer_type){ should == "irrational" }
      its(:r_min){ should == 1  }
      its(:r_max){ should == 11 }
      its(:p_minus){ should be_true }
      its(:p_min){ should == 2  }
      its(:p_max){ should == 21 }
      its(:q_min){ should == 2  }
      its(:q_max){ should == 31 }
    end
    context "set nothing" do
      it "should not raise Exception" do
        lambda{ @basic.set_options({}.to_json) }.should_not raise_exception
      end
    end
  end
end
