require 'spec_helper'

describe SquareRootArithmeticEditForm do
  before do
    @valid_attributes = {
      :unit_id         => 1,
      :level           => 1,
      :amount          => 2,
      :min             => 1,
      :max             => 100,
      :single_term_min => 1,
      :single_term_max => 100,
      :term_number     => 2,
      :operators_plus  => true,
      :operators_minus => true,
      :use_coefficient => true,
      :use_power       => true,
    }
    @klass = SquareRootArithmeticEditForm
    @basic = @klass.new(@valid_attributes)
  end

  context "basic" do
    it{ @basic.should be_valid }
  end

  context "validation" do

    [
     [:min, 0, 1],
     [:max, 0, 1],
     [:single_term_min, 0, 1],
     [:single_term_max, 0, 1],
    ].each do |target, ng , ok|
      context target do
        before do
          @target = target
          @boundary_value_ng = ng
          @boundary_value_ok = ok
        end
        it_should_behave_like 'validates_presence_of'
        it_should_behave_like 'validates_numericality_of'
      end
    end

    context "relation" do
      context "normal" do
        before do
          @target_left = :min
          @target_right = :max
          @message = "Min must be greater than Max."
        end
        it_should_behave_like "check_min_max"
      end

      context "single_term" do
        before do
          @target_left = :single_term_min
          @target_right = :single_term_max
          @message = "Single term min must be greater than Single term max."
        end
        it_should_behave_like "check_min_max"
      end
    end

    context "operators" do
      context "no operators specified" do
        subject{
          @basic.operators_plus = false
          @basic.operators_minus = false
          @basic
        }
        it{ should_not be_valid }
      end
    end
  end

  context "options_hash" do
    subject{ @basic.options_hash }
    it{
      should == {
        :min => 1, :max => 100,
        :single_term_min => 1, :single_term_max => 100,
        :use_coefficient => true, :use_power => true,
        :term_number => 2,
        :operators => %w[+ -]
      }
    }
  end

  context "set_options" do
    context "basic" do
      subject{
        options = {
          :min => 1, :max => 99,
          :single_term_min => 1, :single_term_max => 99,
          :use_coefficient => true, :use_power => true,
          :term_number => 3,
          :operators => %w[+ -]
        }
        @basic.set_options(options.to_json)
        @basic
      }
      its(:min){ should == 1 }
      its(:max){ should == 99 }
      its(:single_term_min){ should == 1 }
      its(:single_term_max){ should == 99 }
      its(:use_coefficient){ should be_true }
      its(:use_power){ should be_true }
      its(:term_number){ should == 3 }
      its(:operators_plus){ should be_true }
      its(:operators_minus){ should be_true }
    end
    context "set nothing" do
      it "should not raise Exception" do
        lambda{ @basic.set_options({}.to_json) }.should_not raise_exception
      end
    end
  end
end
