require 'spec_helper'

describe Unit do
  before(:each) do
    @valid_attributes = {
      :name   => "name",
      :symbol => "symbol"
    }
    @klass = Unit
    @basic = @klass.new(@valid_attributes)
  end

  describe "basic" do
    it{ @basic.should be_valid }
    it{ @basic.problems.should be_empty }
  end

  describe "validation" do
    describe "name" do
      describe "is nil" do
        subject{ @basic.name = nil; @basic }
        it{ should_not be_valid }
      end
      describe "is empty" do
        subject{ @basic.name = ""; @basic }
        it{ should_not be_valid }
      end
      describe "size 40" do
        subject{ @basic.name = "a"*40; @basic }
        it{ should be_valid }
      end
      describe "size 41" do
        subject{ @basic.name = "a"*41; @basic }
        it{ should_not be_valid }
      end
    end
    describe "symbol" do
      describe "is nil" do
        subject{ @basic.symbol = nil; @basic }
        it{ should_not be_valid }
      end
      describe "is empty" do
        subject{ @basic.symbol = ""; @basic }
        it{ should_not be_valid }
      end
      describe "size 40" do
        subject{ @basic.symbol = "a"*40; @basic }
        it{ should be_valid }
      end
      describe "size 41" do
        subject{ @basic.symbol = "a"*41; @basic }
        it{ should_not be_valid }
      end
      describe "invalid format" do
        subject{ @basic.symbol = "A"; @basic }
        it{ should_not be_valid }
      end
      describe "is not unique" do
        before do
          @target = @basic.dup
          @basic.save!
        end
        it{ @target.should_not be_valid }
      end
    end
  end
end
