require 'spec_helper'

describe User do
  before(:each) do
    @valid_attributes = {
      :name => "name",
      :email => "name@example.com",
      :admission_date => Date.today,
    }
    @klass = User
    @basic = @klass.new(@valid_attributes)
  end

  describe "basic" do
    it{ @basic.should be_valid }
  end

  describe "validations" do
    describe "email" do
      describe "is blank" do
        subject{ @basic.email = ""; @basic }
        it{ should be_valid }
      end
      describe "is nil" do
        subject{ @basic.email = nil; @basic }
        it{ should be_valid }
      end
      describe "is bad format" do
        subject{ @basic.email = 'a..b@example.com'; @basic }
        it{ should_not be_valid }
      end
      describe "is not unique" do
        subject{ Factory(:user); @basic }
        it{ should_not be_valid }
      end
    end
  end

  describe "associations" do
    describe "has_many :reports" do
      subject{ Factory(:user_with_reports) }
      its(:reports){ should have(2).reports }
    end
    describe "belongs_to :affiliation" do
      before do
        @target = Factory(:user, :affiliation => Factory(:affiliation))
      end
      it{ @target.affiliation.name.should == "University" }
    end
  end

  describe "callback" do
    describe "before_create" do
      describe "admission_date is nil" do
        before do
          @basic.admission_date = nil
          @basic.save!
          @basic.reload
        end
        it{ @basic.admission_date.should == Date.today }
      end
      describe "update admission_date" do
        before do
          @basic.save!
          @basic.reload
          @basic.update_attributes!(:admission_date => Date.today + 5)
          @basic.reload
        end
        it{ @basic.admission_date.should_not == Date.today }
      end
    end
  end

  describe "role" do
    before do
      @basic.save!
      @basic.has_role! :admin
    end
    it{ @basic.role.should == "admin" }
  end

  describe "have_profile?" do
    describe "with basic" do
      it{ @basic.have_profile?.should_not be_true }
    end
    describe "with modified basic" do
      before do
        @basic.affiliation_id = 1
        @basic.matriculation_number = 1
        @basic.name = 'name'
        @basic.email = 'name@example.com'
      end
      it{ @basic.have_profile?.should be_true }
    end
  end

end
